@@ -547,7 +547,6 @@ static void adjust_fvwm_internal_windows(FvwmWindow *fw)
 	{
 		Scr.Hilite = NULL;
 	}
-	update_last_screen_focus_window(fw);
 	restore_focus_after_unmap(fw, False);
 	frame_destroyed_frame(FW_W(fw));
 	if (fw == Scr.StolenFocusFvwmWin)
@@ -1408,7 +1407,7 @@ static void setup_key_and_button_grabs(FvwmWindow *fw)
 	return;
 }
 
-static void __add_window_handle_x_resources(FvwmWindow *fw)
+static void _add_window_handle_x_resources(FvwmWindow *fw)
 {
 	int client_argc = 0;
 	char **client_argv = NULL;
@@ -1659,6 +1658,7 @@ void setup_title_geometry(
 {
 	int width;
 	int offset;
+	style_flags *sflags = &(pstyle->flags);
 
 	get_title_font_size_and_offset(
 		fw, S_TITLE_DIR(SCF(*pstyle)),
@@ -1669,7 +1669,12 @@ void setup_title_geometry(
 		&width, &offset);
 	fw->title_thickness = width;
 	fw->title_text_offset = offset;
-	fw->corner_width = fw->title_thickness + fw->boundary_width;
+	fw->corner_length = fw->title_thickness + fw->boundary_width;
+	if (SHAS_CORNER_LENGTH(sflags))
+	{
+		fw->corner_length =
+			SGET_CORNER_LENGTH(*pstyle) + fw->boundary_width;
+	}
 	if (!HAS_TITLE(fw))
 	{
 		fw->title_thickness = 0;
@@ -2161,7 +2166,7 @@ FvwmWindow *AddWindow(
 	setup_class_and_resource(fw);
 
 	/****** style setup ******/
-	__add_window_handle_x_resources(fw);
+	_add_window_handle_x_resources(fw);
 	/* get merged styles */
 	lookup_style(fw, &style);
 	sflags = SGET_FLAGS_POINTER(style);
@@ -3301,12 +3306,8 @@ void destroy_window(FvwmWindow *fw)
 	/****** free strings ******/
 
 	free_window_names(fw, True, True);
-
-	if (fw->style_name)
-	{
-		free(fw->style_name);
-		fw->style_name = NULL;
-	}
+	free(fw->style_name);
+	fw->style_name = NULL;
 	if (fw->class.res_name && fw->class.res_name != NoResource)
 	{
 		XFree ((char *)fw->class.res_name);
