@@ -149,7 +149,7 @@ status_send(void)
 
 	m_cur = monitor_get_current();
 
-	cJSON_AddNumberToObject(msg, "version", 2);
+	cJSON_AddNumberToObject(msg, "version", 3);
 	cJSON_AddStringToObject(msg, "current_screen", m_cur->si->name);
 	cJSON_AddStringToObject(msg, "desktop_mode",
 	    monitor_mode == MONITOR_TRACKING_G && is_tracking_shared ? "shared" :
@@ -160,11 +160,14 @@ status_send(void)
 	screens = cJSON_AddObjectToObject(msg, "screens");
 
 	d_count = 0, m_count = 0;
-	TAILQ_FOREACH(m, &monitor_q, entry) {
+	RB_FOREACH(m, monitors, &monitor_q) {
 		cJSON	*this_desktop;
 		if ((desk_doc[d_count] = cJSON_CreateObject()) == NULL)
 			goto out;
 
+		if (m->Desktops == NULL)
+			goto out;
+
 		this_desktop = cJSON_AddObjectToObject(desk_doc[d_count],
 		    "desktops");
 
@@ -192,6 +195,8 @@ status_send(void)
 		}
 
 		cJSON_AddItemToObject(screens, m->si->name, desk_doc[d_count]);
+		cJSON_AddNumberToObject(desk_doc[d_count], "randr_order",
+		    m->number);
 
 		d_count++;
 		m_count++;
@@ -507,7 +512,7 @@ static char *ReadTitleButton(
 }
 
 /* Remove the given decor from all windows */
-static void __remove_window_decors(F_CMD_ARGS, FvwmDecor *d)
+static void _remove_window_decors(F_CMD_ARGS, FvwmDecor *d)
 {
 	const exec_context_t *exc2;
 	exec_context_changes_t ecc;
@@ -868,11 +873,8 @@ static void DestroyFvwmDecor(FvwmDecor *decor)
 	}
 	FreeDecorFace(dpy, &decor->BorderStyle.active);
 	FreeDecorFace(dpy, &decor->BorderStyle.inactive);
-	if (decor->tag)
-	{
-		free(decor->tag);
-		decor->tag = NULL;
-	}
+	free(decor->tag);
+	decor->tag = NULL;
 
 	return;
 }
@@ -1287,7 +1289,7 @@ void update_decors_colorset(int cset)
 	}
 }
 
-static Bool __parse_vector_line_one_coord(
+static Bool _parse_vector_line_one_coord(
 	char **ret_action, int *pcoord, int *poff, char *action)
 {
 	int offset;
@@ -1327,7 +1329,7 @@ static Bool __parse_vector_line_one_coord(
 	return True;
 }
 
-static Bool __parse_vector_line(
+static Bool _parse_vector_line(
 	char **ret_action, int *px, int *py, int *pxoff, int *pyoff, int *pc,
 	char *action)
 {
@@ -1336,7 +1338,7 @@ static Bool __parse_vector_line(
 	int n;
 
 	*ret_action = action;
-	if (__parse_vector_line_one_coord(&action, px, pxoff, action) == False)
+	if (_parse_vector_line_one_coord(&action, px, pxoff, action) == False)
 	{
 		return False;
 	}
@@ -1345,7 +1347,7 @@ static Bool __parse_vector_line(
 		return False;
 	}
 	action++;
-	if (__parse_vector_line_one_coord(&action, py, pyoff, action) == False)
+	if (_parse_vector_line_one_coord(&action, py, pyoff, action) == False)
 	{
 		return False;
 	}
@@ -1692,7 +1694,7 @@ Bool ReadDecorFace(char *s, DecorFace *df, int button, int verbose)
 				int yoff = 0;
 				int c;
 
-				if (__parse_vector_line(
+				if (_parse_vector_line(
 					    &s, &x, &y, &xoff, &yoff, &c, s) ==
 				    False)
 				{
@@ -1838,11 +1840,8 @@ Bool ReadDecorFace(char *s, DecorFace *df, int button, int verbose)
 				}
 				return False;
 			}
-			if (file)
-			{
-				free(file);
-				file = NULL;
-			}
+			free(file);
+			file = NULL;
 
 			memset(&df->style, 0, sizeof(df->style));
 			if (strncasecmp(style,"Tiled",5)==0)
@@ -2251,12 +2250,8 @@ void CMD_CursorMove(F_CMD_ARGS)
 		fvwm_debug(__func__, "CursorMove needs 2 arguments");
 		return;
 	}
-	if (FQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
-			  &x, &y, &JunkX, &JunkY, &JunkMask) == False)
-	{
-		/* pointer is on a different screen */
-		return;
-	}
+	FQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
+			  &x, &y, &JunkX, &JunkY, &JunkMask);
 
 	m = monitor_get_current();
 
@@ -2709,17 +2704,6 @@ void CMD_Wait(F_CMD_ARGS)
 }
 
 void CMD_Quit(F_CMD_ARGS)
-{
-	if (master_pid != getpid())
-	{
-		kill(master_pid, SIGTERM);
-	}
-	Done(0,NULL);
-
-	return;
-}
-
-void CMD_QuitScreen(F_CMD_ARGS)
 {
 	Done(0,NULL);
 
@@ -2892,34 +2876,6 @@ void CMD_ModuleTimeout(F_CMD_ARGS)
 	return;
 }
 
-void CMD_HilightColor(F_CMD_ARGS)
-{
-	char *fore;
-	char *back;
-
-	if (Scr.cur_decor && Scr.cur_decor != &Scr.DefaultDecor)
-	{
-		fvwm_debug(__func__,
-			   "Decors do not support the HilightColor command"
-			   " anymore. Please use"
-			   " 'Style <stylename> HilightFore <forecolor>' and"
-			   " 'Style <stylename> HilightBack <backcolor>' instead."
-			   " Sorry for the inconvenience.");
-		return;
-	}
-	action = GetNextToken(action, &fore);
-	GetNextToken(action, &back);
-	if (fore && back)
-	{
-		xasprintf(&action, "* HilightFore %s, HilightBack %s", fore, back);
-		CMD_Style(F_PASS_ARGS);
-	}
-	free(fore);
-	free(back);
-
-	return;
-}
-
 void CMD_HilightColorset(F_CMD_ARGS)
 {
 	char *newaction;
@@ -3201,7 +3157,7 @@ void CMD_DestroyDecor(F_CMD_ARGS)
 	{
 		if (!do_recreate)
 		{
-			__remove_window_decors(F_PASS_ARGS, found);
+			_remove_window_decors(F_PASS_ARGS, found);
 		}
 		DestroyFvwmDecor(found);
 		if (do_recreate)
@@ -3753,7 +3709,7 @@ static Bool FKeysymToKeycode (Display *disp, KeySym keysym,
 	return False;
 }
 
-static void __fake_event(F_CMD_ARGS, FakeEventType type)
+static void _fake_event(F_CMD_ARGS, FakeEventType type)
 {
 	char *token;
 	char *optlist[] = {
@@ -3848,14 +3804,9 @@ static void __fake_event(F_CMD_ARGS, FakeEventType type)
 				 depth++)
 			{
 				w = child_w;
-				if (FQueryPointer(
+				FQueryPointer(
 						dpy, w, &root, &child_w,
-						&rx, &ry, &x, &y,
-						&JunkMask) == False)
-				{
-					/* pointer is on a different
-					 * screen - that's okay here */
-				}
+						&rx, &ry, &x, &y, &JunkMask);
 			}
 
 			if (type == FakeMouseEvent)
@@ -3940,14 +3891,9 @@ static void __fake_event(F_CMD_ARGS, FakeEventType type)
 			}
 
 			usleep(1000 * val);
-			if (FQueryPointer(
-					dpy, Scr.Root, &root, &JunkRoot,
-					&JunkX, &JunkY, &JunkX, &JunkY,
-					&mask) == False)
-			{
-				/* pointer is on a different screen -
-				 * that's okay here */
-			}
+			FQueryPointer(
+				dpy, Scr.Root, &root, &JunkRoot,
+				&JunkX, &JunkY, &JunkX, &JunkY, &mask);
 			break;
 		case 6:
 		case 7:
@@ -4026,14 +3972,14 @@ static void __fake_event(F_CMD_ARGS, FakeEventType type)
 
 void CMD_FakeClick(F_CMD_ARGS)
 {
-	__fake_event(F_PASS_ARGS, FakeMouseEvent);
+	_fake_event(F_PASS_ARGS, FakeMouseEvent);
 
 	return;
 }
 
 void CMD_FakeKeypress(F_CMD_ARGS)
 {
-	__fake_event(F_PASS_ARGS, FakeKeyEvent);
+	_fake_event(F_PASS_ARGS, FakeKeyEvent);
 
 	return;
 }
@@ -4072,4 +4018,4 @@ void CMD_State(F_CMD_ARGS)
 	}
 
 	return;
-}
+}
\ No newline at end of file
