@@ -73,6 +73,7 @@
 #include "ewmh.h"
 #include "ewmh_intern.h"
 #include "geometry.h"
+#include "virtual.h"
 #include "window_flags.h"
 
 #define EWMH_DEBUG 0
@@ -458,7 +459,7 @@ void *atom_get(Window win, Atom to_get, Atom type, int *size)
 		int asize;
 
 		asize = atom_size(format_ret);
-		data = fxmalloc(num_ret * asize);
+		data = fxmalloc(num_ret * asize + 1);
 		if (format_ret == 32 && asize * 8 != format_ret)
 		{
 			int i;
@@ -472,7 +473,7 @@ void *atom_get(Window win, Atom to_get, Atom type, int *size)
 		{
 			if (data)
 			{
-				memcpy(data, retval, num_ret * asize);
+				memcpy(data, retval, num_ret * asize + 1);
 			}
 		}
 		XFree(retval);
@@ -983,6 +984,7 @@ void EWMH_SetClientListStacking(struct monitor *m)
 
 void ewmh_SetWorkArea(struct monitor *m)
 {
+	struct monitor	*m_global = monitor_get_global();
 	long val[256][4]; /* no more than 256 desktops */
 	int i = 0;
 
@@ -991,17 +993,15 @@ void ewmh_SetWorkArea(struct monitor *m)
 
 	while(i < m->ewmhc.NumberOfDesktops && i < 256)
 	{
-		val[i][0] = m->Desktops->ewmh_working_area.x;
-		val[i][1] = m->Desktops->ewmh_working_area.y;
-		val[i][2] = m->Desktops->ewmh_working_area.width;
-		val[i][3] = m->Desktops->ewmh_working_area.height;
+		val[i][0] = m_global->si->x;
+		val[i][1] = m_global->si->y;
+		val[i][2] = m_global->si->w;
+		val[i][3] = m_global->si->h;
 		i++;
 	}
 	ewmh_ChangeProperty(
 		Scr.Root, "_NET_WORKAREA", EWMH_ATOM_LIST_FVWM_ROOT,
 		(unsigned char *)&val, i*4);
-
-	return;
 }
 
 void ewmh_ComputeAndSetWorkArea(struct monitor *m)
@@ -1013,8 +1013,6 @@ void ewmh_ComputeAndSetWorkArea(struct monitor *m)
 	int x,y,width,height;
 	FvwmWindow *fw;
 
-	/* FIXME: needs broadcast if global monitor in use. */
-
 	for (fw = Scr.FvwmRoot.next; fw != NULL; fw = fw->next)
 	{
 		if (fw->m != m)
@@ -1039,10 +1037,6 @@ void ewmh_ComputeAndSetWorkArea(struct monitor *m)
 
 	if (Scr.bo.do_debug_randr)
 	{
-		fvwm_debug(
-			__func__, "monitor '%s': {l: %d, r: %d, t: %d, b: %d} "
-			"{x: %d, y: %d, w: %d, h: %d}\n", m->si->name,
-			left, right, top, bottom, x, y, width, height);
 	}
 
 	if (
@@ -1058,10 +1052,11 @@ void ewmh_ComputeAndSetWorkArea(struct monitor *m)
 
 		if (Scr.bo.do_debug_randr)
 		{
-			fvwm_debug(__func__, "differ, so setting work area\n");
+			fvwm_debug(__func__, "monitor '%s': "
+			    "{l: %d, r: %d, t: %d, b: %d} "
+			    "{x: %d, y: %d, w: %d, h: %d}\n", m->si->name,
+			    left, right, top, bottom, x, y, width, height);
 		}
-
-		ewmh_SetWorkArea(m);
 	}
 
 	return;
@@ -1076,8 +1071,6 @@ void ewmh_HandleDynamicWorkArea(struct monitor *m)
 	int x,y,width,height;
 	FvwmWindow *fw;
 
-	/* FIXME: needs broadcast if global monitor in use. */
-
 	for (fw = Scr.FvwmRoot.next; fw != NULL; fw = fw->next)
 	{
 		if (fw->m != m)
@@ -1136,18 +1129,6 @@ void EWMH_GetWorkAreaIntersection(
 	int area_h = m->Desktops->ewmh_working_area.height;
 	Bool is_dynamic = False;
 
-	if (Scr.bo.do_debug_randr)
-	{
-		fvwm_debug(
-			__func__, "mon: %s {ax: %d, ay: %d, aw: %d, ah: %d\n",
-			m->si->name, area_x, area_y, area_w, area_h);
-		fvwm_debug(
-			__func__, "mon: %s {x: %d, y: %d, w: %d, h: %d\n",
-			m->si->name, *x, *y, *w, *h);
-	}
-
-	/* FIXME: needs broadcast if global monitor in use. */
-
 	switch(type)
 	{
 	case EWMH_IGNORE_WORKING_AREA:
@@ -1177,10 +1158,9 @@ void EWMH_GetWorkAreaIntersection(
 	*w = nw;
 	*h = nh;
 
-	if (Scr.bo.do_debug_randr)
-	{
-		fvwm_debug(
-			__func__,
+	if (Scr.bo.do_debug_randr &&
+	    (*x != area_x || *y != area_y || *w != area_w || *h != area_h)) {
+		fvwm_debug(__func__,
 			"mon: %s finalising: {x: %d, y: %d, w: %d, h: %d}\n",
 			m->si->name, *x, *y, *w, *h);
 	}
@@ -1223,8 +1203,6 @@ float ewmh_GetStrutIntersection(struct monitor *m,
 	float ret = 0;
 	int x21, y21, x22, y22;
 
-	/* FIXME: possibly need to consider using m->si->x/y/w/h */
-
 	/* left */
 	x21 = 0;
 	y21 = 0;
@@ -1271,8 +1249,6 @@ float EWMH_GetStrutIntersection(struct monitor *m,
 {
 	int left, right, top, bottom;
 
-	/* FIXME: needs broadcast if global monitor in use. */
-
 	left = m->Desktops->ewmh_working_area.x;
 	right = monitor_get_all_widths() -
 		(m->Desktops->ewmh_working_area.x
@@ -1929,7 +1905,7 @@ void EWMH_Init(struct monitor *m)
 	int supported_count = 0;
 	long val;
 	XTextProperty text;
-	unsigned char utf_name[5];
+	char *utf_name = "FVWM3";
 	char *names[1];
 	XClassHint classhints;
 
@@ -1963,19 +1939,23 @@ void EWMH_Init(struct monitor *m)
 		XFree(text.value);
 	}
 
-	/* FVWM in UTF8 */
-	utf_name[0] = 0x46;
-	utf_name[1] = 0x56;
-	utf_name[2] = 0x57;
-	utf_name[3] = 0x4D;
-	utf_name[4] = 0x3;
-
 	ewmh_ChangeProperty(
 		Scr.NoFocusWin, "_NET_WM_NAME", EWMH_ATOM_LIST_PROPERTY_NOTIFY,
-		(unsigned char *)&utf_name, 4);
+		(unsigned char *)utf_name, strlen(utf_name));
 
 	clean_up();
 
+	if (m->Desktops == NULL) {
+		m->Desktops = fxcalloc(1, sizeof *m->Desktops);
+		m->Desktops->name = NULL;
+		m->Desktops->next = NULL;
+		m->Desktops->desk = 0;
+
+		/* XXX: mempcy virtual_scr??? */
+
+		apply_desktops_monitor(m);
+	}
+
 	EWMH_SetDesktopNames(m);
 	EWMH_SetCurrentDesktop(m);
 	EWMH_SetNumberOfDesktops(m);
@@ -1983,9 +1963,6 @@ void EWMH_Init(struct monitor *m)
 	EWMH_SetDesktopGeometry(m);
 	EWMH_SetClientList(m);
 	EWMH_SetClientListStacking(m);
-	ewmh_ComputeAndSetWorkArea(m);
-
-	return;
 }
 
 /*
@@ -2085,4 +2062,4 @@ void EWMH_fullscreen(FvwmWindow *fw)
 	}
 
 	return;
-}
+}
\ No newline at end of file
