@@ -248,7 +248,7 @@ enum
 
 /* ---------------------------- local functions ---------------------------- */
 
-int __eae_parse_range(char *input, int *lower, int *upper)
+int _eae_parse_range(char *input, int *lower, int *upper)
 {
 	int rc;
 	int n;
@@ -307,7 +307,7 @@ static signed int expand_args_extended(
 	int i;
 	size_t len;
 
-	rc = __eae_parse_range(input, &lower, &upper);
+	rc = _eae_parse_range(input, &lower, &upper);
 	if (rc == -1)
 	{
 		return -1;
@@ -492,9 +492,7 @@ static signed int expand_vars_extended(
 		if (string == NULL)
 		{
 			const char *ddn = _("Desk");
-			/* TA:  FIXME!  xasprintf() */
-			allocated_string = fxmalloc(19 + strlen(ddn));
-			sprintf(allocated_string, "%s %i", ddn, cs);
+			xasprintf(&allocated_string, "%s %i", ddn, cs);
 			string = allocated_string;
 		}
 		goto GOT_STRING;
@@ -515,6 +513,15 @@ static signed int expand_vars_extended(
 			goto GOT_STRING;
 		}
 
+		if (strcmp(rest, "prev_primary") == 0) {
+			struct monitor *m2 = monitor_by_last_primary();
+
+			if (m2 != NULL)
+				string = m2->si->name;
+			should_quote = False;
+			goto GOT_STRING;
+		}
+
 		if (strcmp(rest, "current") == 0) {
 			struct monitor *m2 = monitor_get_current();
 
@@ -535,7 +542,7 @@ static signed int expand_vars_extended(
 
 		/* We could be left with "<NAME>.?" */
 		char		*m_name = NULL;
-		struct monitor  *mon2;
+		struct monitor  *mon2 = NULL;
 		char		*rest_s;
 		char *p_free;
 		int got_string;
@@ -556,11 +563,18 @@ static signed int expand_vars_extended(
 				free(p_free);
 				return -1;
 			}
-
 			/* Skip over the monitor name. */
 			rest += strlen(m_name) + 1;
 
 			/* Match remainder to valid fields. */
+			if (strcmp(rest, "name") == 0) {
+				is_numeric = False;
+				string = mon2->si->name;
+				should_quote = False;
+				got_string = 1;
+				break;
+			}
+
 			if (strcmp(rest, "x") == 0) {
 				is_numeric = True;
 				val = mon2->si->x;
@@ -596,6 +610,15 @@ static signed int expand_vars_extended(
 				break;
 			}
 
+			if (strcmp(rest, "number") == 0) {
+				is_numeric = True;
+				val = (int)mon2->number;
+				got_string = 1;
+				fvwm_debug(__func__, "THOMAS: %s %d",
+				mon2->si->name, mon2->number);
+				break;
+			}
+
 			if (strcmp(rest, "desk") == 0) {
 				is_numeric = True;
 				val = mon2->virtual_scr.CurrentDesk;
@@ -1141,12 +1164,8 @@ static signed int expand_vars_extended(
 		case VAR_POINTER_CX:
 			is_x = True;
 		}
-		if (FQueryPointer(dpy, context_w, &JunkRoot, &JunkChild,
-				  &JunkX, &JunkY, &x, &y, &JunkMask) == False)
-		{
-			/* pointer is on a different screen, don't expand */
-			return -1;
-		}
+		FQueryPointer(dpy, context_w, &JunkRoot, &JunkChild,
+				  &JunkX, &JunkY, &x, &y, &JunkMask);
 		val = (is_x) ? x : y;
 		break;
 	case VAR_POINTER_SCREEN:
@@ -1697,4 +1716,4 @@ char *expand_vars(
 	out[j] = 0;
 
 	return out;
-}
+}
\ No newline at end of file
