@@ -347,7 +347,7 @@ static void frame_setup_titlebar(
 	return;
 }
 
-static void __frame_setup_window(
+static void _frame_setup_window(
 	FvwmWindow *fw, rectangle *frame_g, Bool do_send_configure_notify,
 	Bool do_force, Bool is_application_request)
 {
@@ -1535,7 +1535,7 @@ void frame_get_sidebar_geometry(
 	FvwmWindow *fw, DecorFaceStyle *borderstyle, rectangle *frame_g,
 	rectangle *ret_g, Bool *ret_has_x_marks, Bool *ret_has_y_marks)
 {
-	int min_w;
+	int min_l;
 	size_borders b;
 
 	ret_g->x = 0;
@@ -1570,16 +1570,16 @@ void frame_get_sidebar_geometry(
 		*ret_has_x_marks = True;
 		*ret_has_y_marks = True;
 	}
-	ret_g->x = fw->corner_width;
-	ret_g->y = fw->corner_width;
-	min_w = 2 * fw->corner_width + 4;
+	ret_g->x = fw->corner_length;
+	ret_g->y = fw->corner_length;
+	min_l = 2 * fw->corner_length + 4;
 	/* limit by available space, remove handle marks if necessary */
-	if (frame_g->width < min_w)
+	if (frame_g->width < min_l)
 	{
 		ret_g->x = frame_g->width / 3;
 		*ret_has_y_marks = False;
 	}
-	if (frame_g->height < min_w)
+	if (frame_g->height < min_l)
 	{
 		ret_g->y = frame_g->height / 3;
 		*ret_has_x_marks = False;
@@ -2054,7 +2054,7 @@ void frame_setup_window(
 	g.y = y;
 	g.width = w;
 	g.height = h;
-	__frame_setup_window(fw, &g, do_send_configure_notify, False, False);
+	_frame_setup_window(fw, &g, do_send_configure_notify, False, False);
 
 	return;
 }
@@ -2069,7 +2069,7 @@ void frame_setup_window_app_request(
 	g.y = y;
 	g.width = w;
 	g.height = h;
-	__frame_setup_window(fw, &g, do_send_configure_notify, False, True);
+	_frame_setup_window(fw, &g, do_send_configure_notify, False, True);
 
 	return;
 }
@@ -2084,7 +2084,7 @@ void frame_force_setup_window(
 	g.y = y;
 	g.width = w;
 	g.height = h;
-	__frame_setup_window(fw, &g, do_send_configure_notify, True, False);
+	_frame_setup_window(fw, &g, do_send_configure_notify, True, False);
 
 	return;
 }
