@@ -133,8 +133,6 @@ static const char *init_function_names[4] =
 
 /* ---------------------------- exported variables (globals) --------------- */
 
-int master_pid;                 /* process number of 1st fvwm process */
-
 ScreenInfo Scr;                 /* structures for the screen */
 Display *dpy = NULL;            /* which display are we talking to */
 
@@ -492,33 +490,6 @@ static int parse_command_args(
 	return error_code ? error_code : argc;
 }
 
-/*
- *
- */
-static
-char *get_display_name(char *disp_name, int screen_num)
-{
-	char *msg;
-	char *new_dn;
-	char *cp;
-
-	CopyString(&msg, disp_name);
-	cp = strchr(msg, ':');
-	if (cp != NULL)
-	{
-		cp = strchr(cp, '.');
-		if (cp != NULL)
-		{
-			/* truncate at display part */
-			*cp = '\0';
-		}
-	}
-	xasprintf(&new_dn, "%s.%d", msg, screen_num);
-	free(msg);
-
-	return new_dn;
-}
-
 /*
  *
  *  Procedure:
@@ -1175,10 +1146,6 @@ static void InitVariables(void)
 	/* Not the right place for this, should only be called once
 	 * somewhere .. */
 
-	Scr.flags.is_pointer_on_this_screen = !!FQueryPointer(
-		dpy, Scr.Root, &JunkRoot, &JunkChild, &JunkX, &JunkY, &JunkX,
-		&JunkY, &JunkMask);
-
 	/* make sure colorset 0 exists */
 	alloc_colorset(0);
 
@@ -1192,7 +1159,6 @@ static void usage(int is_verbose)
 		   " [-d display]"
 		   " [-f cfgfile]"
 		   " [-c cmd]"
-		   " [-s [screen_num]]"
 		   " [-I vis-id | -C vis-class]"
 		   " [-l colors"
 		   " [-L|A|S|P] ...]"
@@ -1211,7 +1177,7 @@ static void usage(int is_verbose)
 		   " -c cmd:       preprocess configuration file with <cmd>\n"
 		   " -C vis-class: use visual class <vis-class>\n"
 		   " -d display:   run fvwm on <display>\n"
-		   " -D:           enable debug oputput\n"
+		   " -D:           enable debug output\n"
 		   " -f cfgfile:   read configuration from <cfgfile>\n"
 		   " -F file:      used internally for session management\n"
 		   " -h, -?:       print this help message\n"
@@ -1222,7 +1188,6 @@ static void usage(int is_verbose)
                    " -o logfile:   output file or '-' for stderr\n"
 		   " -P:           visual palette\n"
 		   " -r:           replace running window manager\n"
-		   " -s [screen]:  manage a single screen\n"
 		   " -S:           static palette\n"
 		   " -v:           verbose log output\n"
 		   " -V:           print version information\n"
@@ -1321,8 +1286,6 @@ static void SetRCDefaults(void)
 		{ "DefaultFont", "", "" },
 		{ DEFAULT_MENU_STYLE, "", "" },
 		{ "TitleStyle Centered -- Raised", "", "" },
-		{ "Style * Color lightgrey/dimgrey", "", "" },
-		{ "Style * HilightFore black, HilightBack grey", "", "" },
 		{ "AddToFunc LoadDefaultConfig", "", "" },
 		{ "+ I Read "FVWM_DATADIR"/default-config/config", "", "" },
 		{ "+ I StartFunction", "", "" },
@@ -1472,7 +1435,7 @@ void StartupStuff(void)
 	/* Have to do this here too because preprocessor modules have not run
 	 * to the end when HandleEvents is entered from the main loop. */
 	struct monitor *m;
-	TAILQ_FOREACH(m, &monitor_q, entry)
+	RB_FOREACH(m, monitors, &monitor_q)
 		checkPanFrames(m);
 
 	fFvwmInStartup = False;
@@ -1706,8 +1669,6 @@ int main(int argc, char **argv)
 	XSetWindowAttributes attributes;
 	int i;
 	char *display_string;
-	Bool do_force_single_screen = False;
-	int single_screen_num = -1;
 	Bool replace_wm = False;
 	int visualClass = -1;
 	int visualId = -1;
@@ -1813,22 +1774,6 @@ int main(int argc, char **argv)
 			}
 			state_filename = argv[i];
 		}
-		else if (strcmp(argv[i], "-s") == 0 ||
-			 strcmp(argv[i], "-single-screen") == 0 ||
-			 strcmp(argv[i], "--single-screen") == 0)
-		{
-			do_force_single_screen = True;
-			if (i+1 < argc && argv[i+1][0] != '-')
-			{
-				i++;
-				if (sscanf(argv[i], "%d", &single_screen_num) ==
-				    0)
-				{
-					usage(0);
-					exit(1);
-				}
-			}
-		}
 		else if (strcmp(argv[i], "-d") == 0 ||
 			 strcmp(argv[i], "-display") == 0 ||
 			 strcmp(argv[i], "--display") == 0)
@@ -2045,106 +1990,15 @@ int main(int argc, char **argv)
 
 	InstallSignals();
 
-	if (single_screen_num >= 0)
+	if(!(Pdpy = dpy = XOpenDisplay(display_name)))
 	{
-		char *dn = NULL;
-
-		if (display_name)
-		{
-			dn = display_name;
-		}
-		if (!dn)
-		{
-			dn = getenv("DISPLAY");
-		}
-		if (!dn)
-		{
-			/* should never happen ? */
-			if (!(dpy = XOpenDisplay(dn)))
-			{
-				fvwm_debug(__func__, "can't open display %s"
-					   "to get the default display",
-					   XDisplayName(dn));
-			}
-			else
-			{
-				dn = XDisplayString(dpy);
-			}
-		}
-		if (dn == NULL)
-		{
-			fvwm_debug(__func__,
-				   "couldn't find default display (%s)",
-				   XDisplayName(dn));
-		}
-		else
-		{
-			char *new_dn;
-
-			new_dn = get_display_name(dn, single_screen_num);
-			if (dpy && strcmp(new_dn, dn) == 0)
-			{
-				/* allready opened */
-				Pdpy = dpy;
-			}
-			else if (dpy)
-			{
-				XCloseDisplay(dpy);
-				dpy = NULL;
-			}
-			if (!dpy && !(Pdpy = dpy = XOpenDisplay(new_dn)))
-			{
-				fvwm_debug(__func__,
-					   "can't open display %s, single screen "
-					   "number %d maybe not correct",
-					   new_dn, single_screen_num);
-			}
-			Scr.screen = single_screen_num;
-			Scr.NumberOfScreens = ScreenCount(dpy);
-			free(new_dn);
-		}
-	}
-
-	if (!dpy)
-	{
-		if(!(Pdpy = dpy = XOpenDisplay(display_name)))
-		{
-			fvwm_debug(__func__, "can't open display %s",
-				   XDisplayName(display_name));
-			exit (1);
-		}
-		Scr.screen= DefaultScreen(dpy);
-		Scr.NumberOfScreens = ScreenCount(dpy);
+		fvwm_debug(__func__, "can't open display %s",
+			   XDisplayName(display_name));
+		exit (1);
 	}
+	Scr.screen= DefaultScreen(dpy);
 
 	atexit(catch_exit);
-	master_pid = getpid();
-
-	if (!do_force_single_screen)
-	{
-		int myscreen = 0;
-		char *new_dn;
-		char *dn;
-
-		dn = XDisplayString(dpy);
-		for (i=0;i<Scr.NumberOfScreens;i++)
-		{
-			if (i != Scr.screen && fork() == 0)
-			{
-				myscreen = i;
-				new_dn = get_display_name(dn, myscreen);
-				Pdpy = dpy = XOpenDisplay(new_dn);
-				Scr.screen = myscreen;
-				Scr.NumberOfScreens = ScreenCount(dpy);
-				free(new_dn);
-
-				break;
-			}
-		}
-
-		g_argv[argc++] = "-s";
-		g_argv[argc] = NULL;
-	}
 
 	monitor_mode = MONITOR_TRACKING_G;
 	FScreenInit(dpy);
@@ -2389,7 +2243,6 @@ int main(int argc, char **argv)
 			if (prop != NULL)
 			{
 				Restarting = True;
-				/* do_force_single_screen = True; */
 			}
 		}
 	}
@@ -2419,9 +2272,16 @@ int main(int argc, char **argv)
 	LoadWindowStates(state_filename);
 
 	is_tracking_shared = false;
-	TAILQ_FOREACH(m, &monitor_q, entry)
+	RB_FOREACH(m, monitors, &monitor_q) {
 		EWMH_Init(m);
 
+		/* Having initialised the monitor at startup here, we can
+		 * remove this flag, as the monitor is no longer considered
+		 * new.
+		 */
+		m->flags &= ~MONITOR_NEW;
+	}
+
 	SetRCDefaults();
 	flush_window_updates();
 	simplify_style_list();
@@ -2537,4 +2397,4 @@ int main(int argc, char **argv)
 	}
 
 	exit(0);
-}
+}
\ No newline at end of file
