@@ -267,7 +267,7 @@ static void BroadcastNewPacket(unsigned long event_type,
 	return;
 }
 
-action_flags *__get_allowed_actions(const FvwmWindow *fw)
+action_flags *_get_allowed_actions(const FvwmWindow *fw)
 {
 	static action_flags act;
 	act.is_movable = is_function_allowed(
@@ -368,7 +368,7 @@ action_flags *__get_allowed_actions(const FvwmWindow *fw)
 		(unsigned long)(sizeof((*(_fw))->flags)),	\
 		&(*(_fw))->flags,				\
 		(unsigned long)(sizeof(action_flags)),		\
-		__get_allowed_actions((*(_fw)))
+		_get_allowed_actions((*(_fw)))
 
 void SendConfig(fmodule *module, unsigned long event_type, const FvwmWindow *t)
 {
@@ -468,35 +468,47 @@ void BroadcastName(
 void BroadcastMonitorList(fmodule *this)
 {
 	char		*name;
+	const char	*m_info;
 	struct monitor	*m;
 	fmodule_list_itr moditr;
 	fmodule *module;
 
 	module_list_itr_init(&moditr);
 
+	m_info = "Monitor %s %d %d %d %d %d %d %d %d %d %d";
+
 	while ((module = module_list_itr_next(&moditr)) != NULL) {
-		TAILQ_FOREACH(m, &monitor_q, entry) {
-			if (m->flags & MONITOR_DISABLED)
-				continue;
-			xasprintf(&name, "Monitor %s %d %d %d %d %d %d %d %d %d %d %d %d",
-				m->si->name,
-				(int)m->si->rr_output,
-				m == monitor_get_current(),
-				monitor_get_all_widths(),
-				monitor_get_all_heights(),
-				m->virtual_scr.Vx,
-				m->virtual_scr.Vy,
-				m->virtual_scr.VxMax,
-				m->virtual_scr.VyMax,
-				m->si->x,
-				m->si->y,
-				m->si->w,
-				m->si->h
-			);
+		RB_FOREACH(m, monitors, &monitor_q) {
+			xasprintf(&name, m_info, m->si->name, m->flags,
+			    m->dx, m->dy, m->virtual_scr.Vx,
+			    m->virtual_scr.Vy, m->virtual_scr.VxMax,
+			    m->virtual_scr.VyMax, m->virtual_scr.CurrentDesk,
+			    monitor_get_all_widths(), monitor_get_all_heights());
 
 			SendName(module, M_CONFIG_INFO, 0, 0, 0, name);
 			free(name);
 		}
+		xasprintf(&name, "DesktopConfiguration %d %d",
+			monitor_mode, is_tracking_shared);
+		SendName(module, M_CONFIG_INFO, 0, 0, 0, name);
+		free(name);
+
+		/* Reissue the DesktopSize command here, rather than sending
+		 * down the DesktopSize -- we want FvwmPager in particular to
+		 * react to a M_NEW_PAGE event, which DesktopSize will do; and
+		 * this avoids duplication in FvwmPager as a result.
+		 */
+		char action[256];
+
+		/* Every monitor will have the same dx/dy values, so just take
+		 * the fist entry in our list.
+		 */
+		struct monitor *m = RB_MIN(monitors, &monitor_q);
+
+		snprintf(action, sizeof(action), "DesktopSize %dx%d",
+			m->dx, m->dy);
+		execute_function_override_window(NULL, NULL, action, NULL, 0,
+			NULL);
 	}
 }
 
@@ -695,21 +707,10 @@ void module_input_execute(struct fmodule_input *input)
 	 * This is OK now that the Pager uses "Move pointer"
 	 * A real fix would be for the modules to pass the button press coords
 	 */
-	if (FQueryPointer(
+	FQueryPointer(
 		    dpy, Scr.Root, &JunkRoot, &JunkChild, &JunkX,&JunkY,
-		    &e.xbutton.x_root, &e.xbutton.y_root, &e.xbutton.state) ==
-	    False)
-	{
-		/* pointer is not on this screen */
-		/* If a module does XUngrabPointer(), it can now get proper
-		 * Popups */
-		e.xbutton.window = Scr.Root;
-		ecc.w.fw = NULL;
-	}
-	else
-	{
-		e.xbutton.window = input->window;
-	}
+		    &e.xbutton.x_root, &e.xbutton.y_root, &e.xbutton.state);
+	e.xbutton.window = input->window;
 	e.xbutton.subwindow = None;
 	e.xbutton.button = 1;
 	/* If a module does XUngrabPointer(), it can now get proper Popups */
@@ -896,7 +897,7 @@ void CMD_Send_WindowList(F_CMD_ARGS)
 	 * Modules can filter out those monitor packets they're not interested
 	 * in.
 	 */
-	TAILQ_FOREACH(m, &monitor_q, entry) {
+	RB_FOREACH(m, monitors, &monitor_q) {
 		SendPacket(mod, M_NEW_DESK, 2, (long)m->virtual_scr.CurrentDesk,
 			(long)m->si->rr_output);
 		SendPacket(
