@@ -20,6 +20,7 @@
 /* ---------------------------- included header files ---------------------- */
 
 #include "config.h"
+#include "window_flags.h"
 
 #include <stdio.h>
 #ifdef HAVE_GETPWUID
@@ -157,7 +158,7 @@ SaveGlobalState(FILE *f)
 	struct monitor	*m;
 
 	fprintf(f, "[GLOBAL]\n");
-	TAILQ_FOREACH(m, &monitor_q, entry) {
+	RB_FOREACH(m, monitors, &monitor_q) {
 		fprintf(f, "  [MONITOR] %i\n", (int)m->si->rr_output);
 		fprintf(f, "      [DESKTOP] %i\n", m->virtual_scr.CurrentDesk);
 		fprintf(f, "      [VIEWPORT] %i %i %i %i\n",
@@ -745,7 +746,7 @@ set_sm_properties(FSmcConn sm_conn, char *filename, char hint)
 	FSmPropValue prop1val, prop2val, prop3val, prop4val, prop7val;
 	struct passwd *pwd;
 	char *user_id;
-	char *discardCommand;
+	char *discardCommand = NULL;
 	char screen_num[32];
 	int numVals, i, priority = 30;
 	Bool is_xsm_detected = False;
@@ -1053,10 +1054,6 @@ callback_die(FSmcConn sm_conn, FSmPointer client_data)
 	}
 	sm_fd = -1;
 
-	if (master_pid != getpid())
-	{
-		kill(master_pid, SIGTERM);
-	}
 	Done(0, NULL);
 }
 
@@ -1538,6 +1535,8 @@ MatchWinToSM(
 		{
 			matches[i].used = 1;
 
+			SET_ICONIFIED(ewin, 0);
+
 			if (!Restarting)
 			{
 				/* We don't want to restore too much state if
@@ -1609,7 +1608,9 @@ MatchWinToSM(
 					win_opts->initial_icon_x -= m->virtual_scr.Vx;
 					win_opts->initial_icon_y -= m->virtual_scr.Vy;
 				}
-			}
+			} else
+				win_opts->initial_state = NormalState;
+
 			ewin->m = matches[i].m;
 			ewin->g.normal.x = matches[i].x;
 			ewin->g.normal.y = matches[i].y;
@@ -1962,4 +1963,4 @@ void CMD_SaveQuitSession(F_CMD_ARGS)
 	save_quit_session();
 
 	return;
-}
+}
\ No newline at end of file
