@@ -42,6 +42,7 @@
 #include "module_list.h"
 #include "focus.h"
 #include "ewmh.h"
+#include "ewmh_intern.h"
 #include "move_resize.h"
 #include "borders.h"
 #include "frame.h"
@@ -66,7 +67,7 @@
 #define CMD_GOTO_DESK(m, d)						    \
 	do {								    \
 		char	*cmd;						    \
-		xasprintf(&cmd, "GotoDesk %s 0 %d", (m)->si->name, (d));    \
+		xasprintf(&cmd, "GotoDesk screen %s 0 %d", (m)->si->name, (d));    \
 		execute_function_override_window(			    \
 			NULL, NULL, cmd, NULL, 0, NULL);		    \
 		free(cmd);						    \
@@ -77,7 +78,7 @@
 		char	*cmd;						    \
 		xasprintf(&cmd,						    \
 		    "All (!Screen %s, Desk %d, CurrentPage, !CirculateHit) "\
-		    "MoveToPage %s $[w.pagex] $[w.pagey]",		    \
+		    "MoveToPage screen %s $[w.pagex] $[w.pagey]",	    \
 		    (m)->si->name, (d), (m)->si->name);			    \
 		execute_function_override_window(			    \
 			NULL, NULL, cmd, NULL, 0, NULL);		    \
@@ -113,10 +114,6 @@
  *
  * 2 is the default.
  */
-static int edge_thickness = 2;
-static int last_edge_thickness = 2;
-static bool pan_frames_mapped = false;
-
 static void store_desktop_cmd(int, char *);
 static int number_of_desktops(struct monitor *);
 static void init_one_panframe(PanFrame *, int, int, int, int, int);
@@ -137,7 +134,7 @@ static Bool _pred_button_event(Display *display, XEvent *event, XPointer arg)
 		True : False;
 }
 
-static void __drag_viewport(const exec_context_t *exc, int scroll_speed)
+static void _drag_viewport(const exec_context_t *exc, int scroll_speed)
 {
 	XEvent e;
 	int x;
@@ -152,15 +149,9 @@ static void __drag_viewport(const exec_context_t *exc, int scroll_speed)
 		return;
 	}
 
-	if (FQueryPointer(
+	FQueryPointer(
 		    dpy, Scr.Root, &JunkRoot, &JunkChild, &x, &y,
-		    &JunkX, &JunkY, &button_mask) == False)
-	{
-		/* pointer is on a different screen */
-		/* Is this the best thing to do? */
-		UngrabEm(GRAB_NORMAL);
-		return;
-	}
+		    &JunkX, &JunkY, &button_mask);
 	MyXGrabKeyboard(dpy);
 	button_mask &= DEFAULT_ALL_BUTTONS_MASK;
 	memset(&e, 0, sizeof(e));
@@ -203,25 +194,18 @@ static void __drag_viewport(const exec_context_t *exc, int scroll_speed)
 
 			/* Query the pointer to catch the latest information.
 			 * This *is* necessary. */
-			if (FQueryPointer(
+			FQueryPointer(
 				    dpy, Scr.Root, &JunkRoot, &JunkChild, &px,
-				    &py, &JunkX, &JunkY, &JunkMask) == True)
-			{
-				fev_make_null_event(&e2, dpy);
-				e2.type = MotionNotify;
-				e2.xmotion.time = fev_get_evtime();
-				e2.xmotion.x_root = px;
-				e2.xmotion.y_root = py;
-				e2.xmotion.state = JunkMask;
-				e2.xmotion.same_screen = True;
-				e = e2;
-				fev_fake_event(&e);
-			}
-			else
-			{
-				/* pointer is on a different screen,
-				 * ignore event */
-			}
+				    &py, &JunkX, &JunkY, &JunkMask);
+			fev_make_null_event(&e2, dpy);
+			e2.type = MotionNotify;
+			e2.xmotion.time = fev_get_evtime();
+			e2.xmotion.x_root = px;
+			e2.xmotion.y_root = py;
+			e2.xmotion.state = JunkMask;
+			e2.xmotion.same_screen = True;
+			e = e2;
+			fev_fake_event(&e);
 		}
 		/* Handle a limited number of key press events to allow
 		 * mouseless operation */
@@ -629,7 +613,7 @@ static void MapDesk(struct monitor *m, int desk, Bool grab)
 					    m->virtual_scr.CurrentDesk);
 
 					xasprintf(&cmd,
-					    "MoveToPage %s $[w.pagex] $[w.pagey]",
+					    "MoveToPage screen %s $[w.pagex] $[w.pagey]",
 					    m->si->name);
 
 					/* execute_function_override_window()
@@ -734,11 +718,11 @@ int HandlePaging(
 	static Bool is_last_position_valid = False;
 	struct monitor	*m = monitor_get_current();
 	int mwidth, mheight;
+	int edge_thickness = m->virtual_scr.edge_thickness;
 
 	mwidth = monitor_get_all_widths();
 	mheight = monitor_get_all_heights();
 
-
 	delta->x = 0;
 	delta->y = 0;
 	if (!is_timestamp_valid && do_continue_previous)
@@ -789,7 +773,6 @@ int HandlePaging(
 
 	do
 	{
-		int rc;
 		Window JunkW;
 		int JunkC;
 		unsigned int JunkM;
@@ -801,31 +784,10 @@ int HandlePaging(
 			return 0;
 		}
 		/* get pointer location */
-		rc = FQueryPointer(
+		FQueryPointer(
 			dpy, Scr.Root, &JunkW, &JunkW, &x, &y, &JunkC, &JunkC,
 			&JunkM);
-		if (rc == False)
-		{
-			/* pointer is on a different screen */
-			x = 0;
-			y = 0;
-		}
 
-#if 0
-		/* check actual pointer location since PanFrames can get buried
-		 * under window being moved or resized - mab */
-		if (x >= edge_thickness &&
-		    x < mwidth - edge_thickness &&
-		    x <= (m->si->w) - edge_thickness &&
-		    y >= edge_thickness &&
-		    y < mheight - edge_thickness &&
-		    y >= (m->si->h) - edge_thickness)
-		{
-			is_timestamp_valid = False;
-			add_time = 0;
-			return 0;
-		}
-#endif
 		if (!fLoop && is_last_position_valid &&
 		    (x - last_x > MAX_PAGING_MOVE_DISTANCE ||
 		     x - last_x < -MAX_PAGING_MOVE_DISTANCE ||
@@ -857,17 +819,14 @@ int HandlePaging(
 
 	/* Get the latest pointer position.  This is necessary as XFree 4.1.0.1
 	 * sometimes does not report mouse movement when it should. */
-	if (FQueryPointer(
+	FQueryPointer(
 		    dpy, Scr.Root, &JunkRoot, &JunkChild, &x, &y, &JunkX,
-		    &JunkY, &JunkMask) == False)
-	{
-		/* pointer is on a different screen - ignore */
-	}
+		    &JunkY, &JunkMask);
+
 	/* Move the viewport */
 	/* and/or move the cursor back to the approximate correct location */
 	/* that is, the same place on the virtual desktop that it */
 	/* started at */
-
 	if (x <= (m->si->x + edge_thickness))
 	{
 		delta->x = -warp_size.x;
@@ -1004,14 +963,9 @@ int HandlePaging(
 	FWarpPointer(dpy,None,Scr.Root,0,0,0,0,p->x,p->y);
 	MoveViewport(m, m->virtual_scr.Vx + delta->x,
 			m->virtual_scr.Vy + delta->y,False);
-	if (FQueryPointer(
+	FQueryPointer(
 		    dpy, Scr.Root, &JunkRoot, &JunkChild, &(p->x), &(p->y),
-		    &JunkX, &JunkY, &JunkMask) == False)
-	{
-		/* pointer is on a different screen */
-		p->x = 0;
-		p->y = 0;
-	}
+		    &JunkX, &JunkY, &JunkMask);
 	if (Grab)
 	{
 		MyXUngrabServer(dpy);
@@ -1048,6 +1002,8 @@ void checkPanFrames(struct monitor *m)
 	int x = m->si->x;
 	int y = m->si->y;
 
+	int edge_thickness = m->virtual_scr.edge_thickness;
+	int last_edge_thickness = m->virtual_scr.last_edge_thickness;
 
 	if (!Scr.flags.are_windows_captured)
 		return;
@@ -1243,7 +1199,7 @@ void raisePanFrames(void)
 	 * every time they are raised by using XRestackWindows. */
 
 	n = 0;
-	TAILQ_FOREACH(m, &monitor_q, entry) {
+	RB_FOREACH(m, monitors, &monitor_q) {
 		if (m->PanFrameTop.isMapped)
 		{
 			windows[n++] = m->PanFrameTop.win;
@@ -1293,14 +1249,17 @@ init_one_panframe(PanFrame *pf, int x, int y, int w, int h, int cursor)
  * Creates the windows for edge-scrolling
  *
  */
-void initPanFrames(void)
+void initPanFrames(struct monitor *ref)
 {
 	int saved_thickness;
-	struct monitor	*m;
+	int edge_thickness;
+	struct monitor	*m = NULL;
 
-	if (pan_frames_mapped)
+	if (ref == NULL)
 		return;
 
+	edge_thickness = ref->virtual_scr.edge_thickness;
+
 	/* Not creating the frames disables all subsequent behavior */
 	/* TKP. This is bad, it will cause an XMap request on a null window
 	 * later*/
@@ -1312,8 +1271,9 @@ void initPanFrames(void)
 	}
 
 	/* Free panframes here for all monitors. */
-	fvwm_debug(__func__, "freeing panframes");
-	TAILQ_FOREACH(m, &monitor_q, entry) {
+	RB_FOREACH(m, monitors, &monitor_q) {
+		if (ref != NULL && m != ref)
+			continue;
 		if (m->PanFrameLeft.isMapped)
 		{
 			XUnmapWindow(dpy, m->PanFrameLeft.win);
@@ -1336,7 +1296,9 @@ void initPanFrames(void)
 		}
 	}
 
-	TAILQ_FOREACH(m, &monitor_q, entry) {
+	RB_FOREACH(m, monitors, &monitor_q) {
+		if (ref != NULL && m != ref)
+			continue;
 		init_one_panframe(&m->PanFrameTop, m->si->x, m->si->y, m->si->w,
 		    edge_thickness, CRS_TOP_EDGE);
 		init_one_panframe(&m->PanFrameLeft, m->si->x, m->si->y,
@@ -1352,16 +1314,14 @@ void initPanFrames(void)
 			m->PanFrameRight.isMapped= m->PanFrameBottom.isMapped=false;
 		checkPanFrames(m);
 	}
-	edge_thickness = saved_thickness;
-	pan_frames_mapped = true;
-	fvwm_debug(__func__, "finished setting up per-monitor panframes");
+	ref->virtual_scr.edge_thickness = saved_thickness;
 }
 
 Bool is_pan_frame(Window w)
 {
 	struct monitor *m;
 
-	TAILQ_FOREACH(m, &monitor_q, entry) {
+	RB_FOREACH(m, monitors, &monitor_q) {
 		bool is_pf_top = (w == m->PanFrameTop.win);
 		bool is_pf_bottom = (w == m->PanFrameBottom.win);
 		bool is_pf_left = (w == m->PanFrameLeft.win);
@@ -1406,7 +1366,7 @@ static void move_viewport_delta(
 
 	if (do_broadcast)
 	{
-		TAILQ_FOREACH(mloop, &monitor_q, entry)
+		RB_FOREACH(mloop, monitors, &monitor_q)
 		{
 			if (is_tracking_shared) {
 			    if ((!m->virtual_scr.is_swapping ||
@@ -1441,7 +1401,7 @@ static void move_viewport_delta(
 	 * top to bottom and then unmap windows bottom up.
 	 */
 	/* TA: 2020-01-21:  This change of skipping monitors will
-	 * break using 'Scroll' and __drag_viewport().  We need to
+	 * break using 'Scroll' and _drag_viewport().  We need to
 	 * ensure we handle this case properly.
 	 */
 	t = get_next_window_in_stack_ring(&Scr.FvwmRoot);
@@ -1636,7 +1596,7 @@ void MoveViewport(struct monitor *m, int newx, int newy, Bool grab)
 	if (!global)
 		goto done;
 
-	TAILQ_FOREACH(m_loop, &monitor_q, entry) {
+	RB_FOREACH(m_loop, monitors, &monitor_q) {
 		if (m == m_loop)
 			continue;
 		checkPanFrames(m_loop);
@@ -1718,7 +1678,7 @@ void goto_desk(int desk, struct monitor *m)
 	monitor_assign_virtual(m);
 
 	if (monitor_mode == MONITOR_TRACKING_G && !is_tracking_shared) {
-		TAILQ_FOREACH(m2, &monitor_q, entry) {
+		RB_FOREACH(m2, monitors, &monitor_q) {
 			if (m == m2)
 				continue;
 			m2->Desktops = m->Desktops;
@@ -1737,7 +1697,7 @@ void goto_desk(int desk, struct monitor *m)
 	if (!is_tracking_shared)
 		return;
 
-	TAILQ_FOREACH(m2, &monitor_q, entry) {
+	RB_FOREACH(m2, monitors, &monitor_q) {
 		/* If we're swapping a desktop between monitors for
 		 * shared mode, only do this when the is_swapping flag
 		 * is true, otherwise events would be raised for a
@@ -1816,28 +1776,30 @@ Bool get_page_arguments(
 	int mw, mh;
 	char *token;
 	char *taction;
-	char *next;
 	int wrapx;
 	int wrapy;
 	int limitdeskx;
 	int limitdesky;
-	struct monitor	*m;
+	struct monitor	*m = NULL;
 
 	wrapx = 0;
 	wrapy = 0;
 	limitdeskx = 1;
 	limitdesky = 1;
 
-	token = PeekToken(action, &next);
-	m = monitor_resolve_name(token);
-	if (m != NULL)
-	{
-		action = next;
-	}
-	else
-	{
-		m = (fw && fw->m) ? fw->m : monitor_get_current();
+	token = PeekToken(action, NULL);
+	if (StrEquals(token, "screen")) {
+		/* Skip literal 'screen' */
+		token = PeekToken(action, &action);
+		/* Actually get the screen value. */
+		token = PeekToken(action, &action);
+
+		if ((m = monitor_resolve_name(token)) == NULL)
+			fvwm_debug(__func__, "Invalid screen: %s", token);
 	}
+
+	if (m == NULL)
+		m = monitor_get_current();
 	*mret = m;
 
 	mw = monitor_get_all_widths();
@@ -2048,9 +2010,9 @@ void parse_edge_leave_command(char *action, int type)
 	if ((option = PeekToken(action, NULL)) != NULL)
 		command = action;
 
-	initPanFrames();
+	initPanFrames(NULL);
 
-	TAILQ_FOREACH(m_loop, &monitor_q, entry) {
+	RB_FOREACH(m_loop, monitors, &monitor_q) {
 		char **target;
 
 		/* Skip this monitor if it's not the one we're after. */
@@ -2113,6 +2075,23 @@ void CMD_EdgeLeaveCommand(F_CMD_ARGS)
 void CMD_EdgeThickness(F_CMD_ARGS)
 {
 	int val, n;
+	char *option;
+	struct monitor *m = NULL, *m_loop = NULL;
+
+	option = PeekToken(action, NULL);
+	if (StrEquals(option, "screen")) {
+		/* Skip literal 'screen' */
+		option = PeekToken(action, &action);
+		/* Actually get the screen value. */
+		option = PeekToken(action, &action);
+
+		if ((m = monitor_resolve_name(option)) == NULL) {
+			fvwm_debug(__func__, "Invalid screen: %s", option);
+			return;
+		}
+
+		fvwm_debug(__func__, "Only setting for %s", m->si->name);
+	}
 
 	n = GetIntegerArguments(action, NULL, &val, 1);
 	if (n != 1)
@@ -2130,17 +2109,43 @@ void CMD_EdgeThickness(F_CMD_ARGS)
 			   " found %d",val);
 		return;
 	}
-	edge_thickness = val;
-	initPanFrames();
 
-	return;
+	/* If we've only been asked to change this for one monitor, stop doing
+	 * anything else after this point, so as not to overwrite any other
+	 * per-monitor values.
+	 */
+	if (m != NULL) {
+		m->virtual_scr.edge_thickness = val;
+		initPanFrames(m);
+
+		return;
+	}
+
+	RB_FOREACH(m_loop, monitors, &monitor_q) {
+		m_loop->virtual_scr.edge_thickness = val;
+		initPanFrames(m_loop);
+	}
 }
 
 void CMD_EdgeScroll(F_CMD_ARGS)
 {
 	int val1, val2, val1_unit, val2_unit, n;
-	char *token;
-	struct monitor	*m;
+	char *token, *option;
+	struct monitor	*m = NULL, *m_this = NULL;
+
+	option = PeekToken(action, NULL);
+	if (StrEquals(option, "screen")) {
+		/* Skip literal 'screen' */
+		option = PeekToken(action, &action);
+		/* Actually get the screen value. */
+		option = PeekToken(action, &action);
+
+		if ((m = monitor_resolve_name(option)) == NULL) {
+			fvwm_debug(__func__, "Invalid screen: %s", option);
+			return;
+		}
+	}
+
 
 	n = GetTwoArguments(action, &val1, &val2, &val1_unit, &val2_unit);
 	if (n != 2)
@@ -2193,14 +2198,19 @@ void CMD_EdgeScroll(F_CMD_ARGS)
 		}
 	}
 
-	TAILQ_FOREACH(m, &monitor_q, entry) {
+	if (m != NULL) {
 		m->virtual_scr.EdgeScrollX = val1 * val1_unit / 100;
 		m->virtual_scr.EdgeScrollY = val2 * val2_unit / 100;
 		checkPanFrames(m);
+
+		return;
 	}
 
-
-	return;
+	RB_FOREACH(m_this, monitors, &monitor_q) {
+		m_this->virtual_scr.EdgeScrollX = val1 * val1_unit / 100;
+		m_this->virtual_scr.EdgeScrollY = val2 * val2_unit / 100;
+		checkPanFrames(m_this);
+	}
 }
 
 void CMD_EdgeResistance(F_CMD_ARGS)
@@ -2290,10 +2300,10 @@ void CMD_DesktopConfiguration(F_CMD_ARGS)
 		 */
 		is_tracking_shared = false;
 		char *cmd = NULL;
-		TAILQ_FOREACH(m_loop, &monitor_q, entry) {
+		RB_FOREACH(m_loop, monitors, &monitor_q) {
 			if (m_loop == m)
 				continue;
-			xasprintf(&cmd, "GotoDeskAndPage %s %d %d %d",
+			xasprintf(&cmd, "GotoDeskAndPage screen %s %d %d %d",
 				m_loop->si->name,
 				m->virtual_scr.CurrentDesk,
 				m->virtual_scr.Vx / monitor_get_all_widths(),
@@ -2374,7 +2384,7 @@ void CMD_DesktopConfiguration(F_CMD_ARGS)
 			 * increment the desktop they're on, having first set
 			 * the tracking mode to be per-monitor.
 			 */
-			struct monitor	*m_first = TAILQ_FIRST(&monitor_q);
+			struct monitor	*m_first = RB_MIN(monitors, &monitor_q);
 			struct monitor	*m_loop;
 			int		 this_d = m_first->virtual_scr.CurrentDesk;
 
@@ -2384,7 +2394,7 @@ void CMD_DesktopConfiguration(F_CMD_ARGS)
 			 * Go through the other monitors, and assign them a
 			 * new desk which is higher than the previous.
 			 */
-			TAILQ_FOREACH(m_loop, &monitor_q, entry) {
+			RB_FOREACH(m_loop, monitors, &monitor_q) {
 				if (m_loop == m_first) {
 					this_d++;
 					continue;
@@ -2410,10 +2420,10 @@ void CMD_DesktopConfiguration(F_CMD_ARGS)
 			 * be moved to the screen for which the desktop has
 			 * the current desktop.
 			 */
-			struct monitor	*m_loop, *first = TAILQ_FIRST(&monitor_q);
+			struct monitor	*m_loop, *first = RB_MIN(monitors, &monitor_q);
 			int		 this_desk = first->virtual_scr.CurrentDesk;
 
-			TAILQ_FOREACH(m_loop, &monitor_q, entry) {
+			RB_FOREACH(m_loop, monitors, &monitor_q) {
 				if (m_loop == first) {
 					CMD_MOVE_SCREEN_DESK(m_loop,
 					    m_loop->virtual_scr.CurrentDesk);
@@ -2432,8 +2442,9 @@ void CMD_DesktopConfiguration(F_CMD_ARGS)
 	return;
 
 update:
-	initPanFrames();
+	initPanFrames(NULL);
 	raisePanFrames();
+	ewmh_SetWorkArea(m);
 
 	for (t = Scr.FvwmRoot.next; t; t = t->next)
 		update_fvwm_monitor(t);
@@ -2465,7 +2476,7 @@ void CMD_DesktopSize(F_CMD_ARGS)
 		return;
 	}
 
-	TAILQ_FOREACH(m, &monitor_q, entry) {
+	RB_FOREACH(m, monitors, &monitor_q) {
 		m->dx = val[0] <= 0 ? 1 : val[0];
 		m->dy = val[1] <= 0 ? 1 : val[1];
 
@@ -2494,20 +2505,23 @@ void CMD_DesktopSize(F_CMD_ARGS)
  */
 void CMD_GotoDesk(F_CMD_ARGS)
 {
-	struct monitor  *m, *m_loop;
-	char		*next, *token;
+	struct monitor  *m = NULL, *m_loop;
+	char		*token;
 	int		 new_desk;
 
-	token = PeekToken(action, &next);
-	m = monitor_resolve_name(token);
-	if (m != NULL)
-	{
-		action = next;
+	token = PeekToken(action, NULL);
+	if (StrEquals(token, "screen")) {
+		/* Skip literal 'screen' */
+		token = PeekToken(action, &action);
+		/* Actually get the screen value. */
+		token = PeekToken(action, &action);
+
+		if ((m = monitor_resolve_name(token)) == NULL)
+			fvwm_debug(__func__, "Invalid screen: %s", token);
 	}
-	else
-	{
+
+	if (m == NULL)
 		m = monitor_get_current();
-	}
 
 	new_desk = GetDeskNumber(m, action, m->virtual_scr.CurrentDesk);
 
@@ -2520,7 +2534,7 @@ void CMD_GotoDesk(F_CMD_ARGS)
 		 */
 		int	 this_desk_now = m->virtual_scr.CurrentDesk;
 
-		TAILQ_FOREACH(m_loop, &monitor_q, entry) {
+		RB_FOREACH(m_loop, monitors, &monitor_q) {
 			if (m_loop == m)
 				continue;
 
@@ -2563,22 +2577,24 @@ void CMD_Desk(F_CMD_ARGS)
 void CMD_GotoDeskAndPage(F_CMD_ARGS)
 {
 	int val[3];
+	int current_desk;
 	Bool is_new_desk;
-	char *next;
 	char *token;
-	struct monitor  *m;
+	struct monitor  *m = NULL;
 
-	token = PeekToken(action, &next);
-	m = monitor_resolve_name(token);
-	if (m != NULL)
-	{
-		action = next;
+	token = PeekToken(action, NULL);
+	if (StrEquals(token, "screen")) {
+		/* Skip literal 'screen' */
+		token = PeekToken(action, &action);
+		/* Actually get the screen value. */
+		token = PeekToken(action, &action);
+
+		if ((m = monitor_resolve_name(token)) == NULL)
+			fvwm_debug(__func__, "Invalid screen: %s", token);
 	}
-	else
-	{
+
+	if (m == NULL)
 		m = monitor_get_current();
-	}
-	/* FIXME: monitor needs broadcast when global. */
 
 	if (MatchToken(action, "prev"))
 	{
@@ -2596,6 +2612,7 @@ void CMD_GotoDeskAndPage(F_CMD_ARGS)
 		return;
 	}
 
+	current_desk = m->virtual_scr.CurrentDesk;
 	is_new_desk = (m->virtual_scr.CurrentDesk != val[0]);
 
 	if (is_tracking_shared && is_new_desk) {
@@ -2604,14 +2621,11 @@ void CMD_GotoDeskAndPage(F_CMD_ARGS)
 		UnmapDesk(m, m->virtual_scr.CurrentDesk, True);
 		CMD_GOTO_DESK(m, val[0]);
 	}
-	m->virtual_scr.prev_desk_and_page_page_x = m->virtual_scr.Vx;
-	m->virtual_scr.prev_desk_and_page_page_y = m->virtual_scr.Vy;
 	MoveViewport(m, val[1], val[2], True);
 	if (is_new_desk)
 	{
-		m->virtual_scr.prev_desk = m->virtual_scr.CurrentDesk;
-		m->virtual_scr.prev_desk_and_page_desk = m->virtual_scr.CurrentDesk;
-		m->virtual_scr.CurrentDesk = val[0];
+		m->virtual_scr.prev_desk = current_desk;
+		m->virtual_scr.prev_desk_and_page_desk = current_desk;
 
 		/* If we're in global mode, store the virtual_scr across all
 		 * monitors.  This will allow for per-monitor tracking to be
@@ -2630,8 +2644,6 @@ void CMD_GotoDeskAndPage(F_CMD_ARGS)
 		if (is_tracking_shared)
 			goto done;
 
-		BroadcastPacket(M_NEW_DESK, 2, (long)m->virtual_scr.CurrentDesk,
-			(long)m->si->rr_output);
 		/* FIXME: domivogt (22-Apr-2000): Fake a 'restack' for sticky
 		 * window upon desk change.  This is a workaround for a
 		 * problem in FvwmPager: The pager has a separate 'root'
@@ -2643,13 +2655,7 @@ void CMD_GotoDeskAndPage(F_CMD_ARGS)
 		 * pager doesn't the stacking order. */
 		BroadcastRestackAllWindows();
 	}
-	else
-	{
-		BroadcastPacket(M_NEW_DESK, 2, (long)m->virtual_scr.CurrentDesk,
-				(long)m->si->rr_output);
-	}
 done:
-	BroadcastMonitorList(NULL);
 	EWMH_SetCurrentDesktop(m);
 
 	return;
@@ -2711,10 +2717,10 @@ void CMD_MoveToDesk(F_CMD_ARGS)
 		struct monitor	*m;
 		char		*cmd;
 
-		TAILQ_FOREACH(m, &monitor_q, entry) {
+		RB_FOREACH(m, monitors, &monitor_q) {
 			if (m->virtual_scr.CurrentDesk == desk) {
 				xasprintf(&cmd,
-				    "MoveToPage %s $[w.pagex] $[w.pagey]",
+				    "MoveToPage screen %s $[w.pagex] $[w.pagey]",
 				    m->si->name);
 				execute_function_override_window(
 					NULL, NULL, cmd, NULL, 0, fw);
@@ -2762,7 +2768,7 @@ void CMD_Scroll(F_CMD_ARGS)
 				scroll_speed *= -1;
 			}
 		}
-		__drag_viewport(exc, scroll_speed);
+		_drag_viewport(exc, scroll_speed);
 
 		return;
 	}
@@ -3016,11 +3022,8 @@ apply_desktops_monitor(struct monitor *m)
 
 		if (d != NULL)
 		{
-			if (d->name != NULL)
-			{
-				free(d->name);
-				d->name = NULL;
-			}
+			free(d->name);
+			d->name = NULL;
 			CopyString(&d->name, dc->name);
 		}
 		else
@@ -3103,6 +3106,6 @@ void CMD_DesktopName(F_CMD_ARGS)
 
 	store_desktop_cmd(desk, action);
 
-	TAILQ_FOREACH(m, &monitor_q, entry)
+	RB_FOREACH(m, monitors, &monitor_q)
 		apply_desktops_monitor(m);
 }
