@@ -15,6 +15,8 @@
 
 #include "config.h"
 
+#include <X11/extensions/Xrandr.h>
+#include <X11/extensions/randr.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdbool.h>
@@ -22,12 +24,15 @@
 #include <errno.h>
 #include <err.h>
 
+#include "log.h"
 #include "defaults.h"
 #include "fvwmlib.h"
 #include "Parse.h"
 #include "PictureBase.h"
 #include "FScreen.h"
 #include "FEvent.h"
+#include "queue.h"
+#include "strtonum.h"
 
 #define GLOBAL_SCREEN_NAME "_global"
 
@@ -46,16 +51,26 @@ static bool		 is_randr_present;
 static bool		 randr_initialised;
 
 static void		 scan_screens(Display *);
+static struct monitor	*monitor_new(const char *);
+static void		 monitor_mark_new(struct monitor *);
+static void		 monitor_mark_inlist(const char *);
+static bool		 monitor_mark_changed(struct monitor *, XRRMonitorInfo *);
 static struct monitor	*monitor_by_name(const char *);
+static void		 monitor_set_coords(struct monitor *, XRRMonitorInfo);
+static void		 monitor_assign_number(void);
+static void		 monitor_add(XRRMonitorInfo *);
 
 enum monitor_tracking monitor_mode;
 bool			 is_tracking_shared;
-struct screen_infos	 screen_info_q;
-struct monitors		monitor_q;
+struct screen_infos	 screen_info_q, screen_info_q_temp;
+struct monitors		 monitor_q;
+struct monitorsold	 monitorsold_q;
 int randr_event;
 const char *prev_focused_monitor;
 static struct monitor	*monitor_global = NULL;
 
+RB_GENERATE(monitors, monitor, entry, monitor_compare);
+
 static void GetMouseXY(XEvent *eventp, int *x, int *y)
 {
 	XEvent e;
@@ -69,12 +84,15 @@ static void GetMouseXY(XEvent *eventp, int *x, int *y)
 		disp, DefaultRootWindow(disp), eventp, x, y);
 }
 
-struct monitor *
-monitor_new(void)
+static struct monitor *
+monitor_new(const char *name)
 {
 	struct monitor	*m;
 
 	m = fxcalloc(1, sizeof *m);
+	m->flags = 0;
+	m->si = screen_info_new();
+	m->si->name = fxstrdup(name);
 
 	return (m);
 }
@@ -90,7 +108,7 @@ monitor_scan_edges(struct monitor *m)
 	m->edge.top = m->edge.bottom = m->edge.left = m->edge.right =
 		MONITOR_OUTSIDE_EDGE;
 
-	TAILQ_FOREACH(m_loop, &monitor_q, entry) {
+	RB_FOREACH(m_loop, monitors, &monitor_q) {
 		if (m_loop == m)
 			continue;
 
@@ -109,6 +127,18 @@ monitor_scan_edges(struct monitor *m)
 	}
 }
 
+struct monitor *
+monitor_by_number(int n)
+{
+	struct monitor	*m_loop;
+
+	RB_FOREACH(m_loop, monitors, &monitor_q) {
+		if (m_loop->number == n)
+			return (m_loop);
+	}
+	return (NULL);
+}
+
 void
 monitor_refresh_global(void)
 {
@@ -121,11 +151,8 @@ monitor_refresh_global(void)
 	 *   si->name,
 	 *   si->{x, y, w, h}
 	 */
-	if (monitor_global == NULL) {
-		monitor_global = monitor_new();
-		monitor_global->si = screen_info_new();
-		monitor_global->si->name = fxstrdup(GLOBAL_SCREEN_NAME);
-	}
+	if (monitor_global == NULL)
+		monitor_global = monitor_new(GLOBAL_SCREEN_NAME);
 
 	/* At this point, the global screen has been initialised.  Refresh the
 	 * coordinate list.
@@ -173,7 +200,7 @@ monitor_get_prev(void)
 	struct monitor	*m, *mret = NULL;
 	struct monitor  *this = monitor_get_current();
 
-	TAILQ_FOREACH(m, &monitor_q, entry) {
+	RB_FOREACH(m, monitors, &monitor_q) {
 		if (m == this)
 			continue;
 		if (m->is_prev) {
@@ -212,15 +239,25 @@ struct monitor *
 monitor_resolve_name(const char *scr)
 {
 	struct monitor	*m = NULL;
+	int		 pos = -1;
+	const char	*errstr;
 
 	if (scr == NULL)
 	{
 		return NULL;
 	}
+
+	/* Try and look up the monitor as a number.  If that succeeds,
+	 * try and match that number as something valid in the monitor
+	 * information we have.
+	 */
+	pos = strtonum(scr, 0, INT_MAX, &errstr);
+	if (errstr == NULL)
+		return (monitor_by_number(pos));
+
 	/* "@g" is for the global screen. */
 	if (strcmp(scr, "g") == 0) {
-		monitor_refresh_global();
-		m = monitor_global;
+		m = monitor_get_global();
 	}
 	/* "@c" is for the current screen. */
 	else if (strcmp(scr, "c") == 0)
@@ -248,7 +285,7 @@ monitor_by_name(const char *name)
 	}
 
 	mret = NULL;
-	TAILQ_FOREACH(m, &monitor_q, entry) {
+	RB_FOREACH(m, monitors, &monitor_q) {
 		if (strcmp(m->si->name, name) == 0) {
 			mret = m;
 			break;
@@ -263,7 +300,7 @@ monitor_by_output(int output)
 {
 	struct monitor	*m, *mret = NULL;
 
-	TAILQ_FOREACH(m, &monitor_q, entry) {
+	RB_FOREACH(m, monitors, &monitor_q) {
 		if (m->si->rr_output == output) {
 		       mret = m;
 		       break;
@@ -274,7 +311,7 @@ monitor_by_output(int output)
 		fvwm_debug(__func__,
 			   "%s: couldn't find monitor with output '%d', "
 			   "returning first output.\n", __func__, output);
-		mret = TAILQ_FIRST(&monitor_q);
+		mret = RB_MIN(monitors, &monitor_q);
 	}
 
 	return (mret);
@@ -285,7 +322,7 @@ monitor_by_primary(void)
 {
 	struct monitor	*m = NULL, *m_loop;
 
-	TAILQ_FOREACH(m_loop, &monitor_q, entry) {
+	RB_FOREACH(m_loop, monitors, &monitor_q) {
 		if (m_loop->flags & MONITOR_PRIMARY) {
 			m = m_loop;
 			break;
@@ -295,13 +332,39 @@ monitor_by_primary(void)
 	return (m);
 }
 
+struct monitor *
+monitor_by_last_primary(void)
+{
+	struct monitor	*m = NULL, *m_loop;
+
+	RB_FOREACH(m_loop, monitors, &monitor_q) {
+		if (m_loop->was_primary && !(m_loop->flags & MONITOR_PRIMARY)) {
+			m = m_loop;
+			break;
+		}
+	}
+
+	return (m);
+}
+
+int changed_monitor_count(void)
+{
+	struct monitor	*m;
+	int c = 0;
+
+	TAILQ_FOREACH(m, &monitorsold_q, oentry)
+		c++;
+
+	return (c);
+}
+
 static void
 monitor_check_primary(void)
 {
 	if (monitor_by_primary() == NULL) {
 		struct monitor	*m;
 
-		m = TAILQ_FIRST(&monitor_q);
+		m = RB_MIN(monitors, &monitor_q);
 		m->flags |= MONITOR_PRIMARY;
 	}
 }
@@ -326,7 +389,7 @@ monitor_assign_virtual(struct monitor *ref)
 	if (monitor_mode == MONITOR_TRACKING_M || is_tracking_shared)
 		return;
 
-	TAILQ_FOREACH(m, &monitor_q, entry) {
+	RB_FOREACH(m, monitors, &monitor_q) {
 		if (m == ref)
 			continue;
 
@@ -340,152 +403,228 @@ monitor_assign_virtual(struct monitor *ref)
 void
 FScreenSelect(Display *dpy)
 {
-	XRRSelectInput(disp, DefaultRootWindow(disp),
+	XRRSelectInput(dpy, DefaultRootWindow(dpy),
 		RRScreenChangeNotifyMask | RROutputChangeNotifyMask);
 }
 
 void
 monitor_output_change(Display *dpy, XRRScreenChangeNotifyEvent *e)
 {
-	XRRScreenResources	*res;
-	struct monitor		*m = NULL;
+	scan_screens(dpy);
+}
 
-	fvwm_debug(__func__, "%s: outputs have changed\n", __func__);
+static void
+monitor_set_coords(struct monitor *m, XRRMonitorInfo rrm)
+{
+	if (m == NULL)
+		return;
 
-	if ((res = XRRGetScreenResources(dpy, e->root)) == NULL) {
-		fvwm_debug(__func__,
-			   "%s: couldn't acquire screen resources\n",
-			   __func__);
+	m->si->x = rrm.x;
+	m->si->y = rrm.y;
+	m->si->w = rrm.width;
+	m->si->h = rrm.height;
+	m->si->rr_output = *rrm.outputs;
+	if (rrm.primary > 0) {
+		m->flags |= MONITOR_PRIMARY;
+		m->was_primary = false;
+	} else {
+		if (m->flags & MONITOR_PRIMARY) {
+			m->flags &= ~MONITOR_PRIMARY;
+			m->was_primary = true;
+		}
+	}
+}
+
+static void
+monitor_assign_number(void)
+{
+	struct monitor	*m;
+	int		 n = 0;
+
+	RB_FOREACH(m, monitors, &monitor_q) {
+		if (m->flags & MONITOR_DISABLED)
+			continue;
+		m->number = n++;
+	}
+}
+
+int
+monitor_compare(struct monitor *a, struct monitor *b)
+{
+	return (a->si->y == b->si->y) ?
+		(a->si->x - b->si->x) : (a->si->y - b->si->y);
+}
+
+static void
+monitor_mark_new(struct monitor *m)
+{
+	if (m == NULL)
+		return;
+
+	m->flags |= MONITOR_NEW;
+
+	memset(&m->virtual_scr, 0, sizeof(m->virtual_scr));
+	m->virtual_scr.edge_thickness = 2;
+	m->virtual_scr.last_edge_thickness = 2;
+
+	memset(&m->ewmhc, 0, sizeof(m->ewmhc));
+	m->Desktops = NULL;
+	m->is_prev = false;
+	m->was_primary = false;
+
+	TAILQ_INSERT_TAIL(&screen_info_q, m->si, entry);
+
+	fvwm_debug(__func__, "Added new monitor: %s (%p)", m->si->name, m);
+}
+
+static void
+monitor_add(XRRMonitorInfo *rrm)
+{
+	struct monitor	*m = NULL;
+	char		*name = NULL;
+
+	if ((name = XGetAtomName(disp, rrm->name)) == NULL) {
+		fvwm_debug(__func__, "Tried detecting monitor "
+		    "with output '%d' but it has no name.  Skipping.",
+		    (int)*rrm->outputs);
 		return;
 	}
+	m = monitor_new(name);
+	monitor_set_coords(m, *rrm);
+	monitor_mark_new(m);
 
-	{
-		struct screen_info	*si;
-		XRROutputInfo		*oinfo = NULL;
-		int			 i;
+	XFree(name);
 
-		scan_screens(dpy);
+	RB_INSERT(monitors, &monitor_q, m);
+}
 
-		for (i = 0; i < res->noutput; i++) {
-			oinfo = XRRGetOutputInfo(dpy, res, res->outputs[i]);
-			if (oinfo == NULL)
-				continue;
+static bool
+monitor_mark_changed(struct monitor *m, XRRMonitorInfo *rrm)
+{
+	if (m == NULL || rrm == NULL)
+		return (false);
 
-			si = screen_info_by_name(oinfo->name);
+	if (m->si->x != rrm->x ||
+	    m->si->y != rrm->y ||
+	    m->si->w != rrm->width ||
+	    m->si->h != rrm->height) {
+		m->flags |= MONITOR_CHANGED;
+		m->emit |= MONITOR_CHANGED;
 
-			if (si == NULL)
-				continue;
+		fvwm_debug(__func__, "%s: x: %d, y: %d, w: %d, h: %d "
+			"comp: x: %d, y: %d, w: %d, h: %d\n",
+			m->si->name, m->si->x, m->si->y, m->si->w, m->si->h,
+			rrm->x, rrm->y, rrm->width, rrm->height);
 
-			TAILQ_FOREACH(m, &monitor_q, entry) {
-				if (m->si == si) {
-					m->emit &= ~MONITOR_ALL;
-					break;
-				}
-			}
+		monitor_set_coords(m, *rrm);
+		TAILQ_INSERT_TAIL(&monitorsold_q, m, oentry);
 
-			if (m == NULL)
-				continue;
-
-			if (oinfo->connection == RR_Connected &&
-			    m->flags & MONITOR_ENABLED) {
-				if (m->flags & MONITOR_CHANGED)
-					m->emit |= MONITOR_CHANGED;
-				continue;
-			}
-
-			if (oinfo->connection == RR_Disconnected &&
-			    m->flags & MONITOR_DISABLED) {
-				continue;
-			}
-
-			switch (oinfo->connection) {
-			case RR_Connected:
-				m->flags &= ~MONITOR_DISABLED;
-				m->flags |= MONITOR_ENABLED;
-
-				m->emit |= MONITOR_ENABLED;
-				break;
-			case RR_Disconnected:
-				m->flags &= ~MONITOR_ENABLED;
-				m->flags |= MONITOR_DISABLED;
-				m->emit |= MONITOR_DISABLED;
-				break;
-			default:
-				break;
-			}
-		}
-		XRRFreeOutputInfo(oinfo);
+		return (true);
 	}
-	XRRFreeScreenResources(res);
+	return (false);
+}
 
-	TAILQ_FOREACH(m, &monitor_q, entry)
-		monitor_scan_edges(m);
+static void
+monitor_mark_inlist(const char *name)
+{
+	struct monitor	 *m = NULL;
 
-	monitor_check_primary();
+	if (name == NULL)
+		return;
+
+	RB_FOREACH(m, monitors, &monitor_q) {
+		if (strcmp(m->si->name, name) == 0) {
+			m->flags |= MONITOR_FOUND;
+			break;
+		}
+	}
 }
 
 static void
 scan_screens(Display *dpy)
 {
+	struct monitor		*m = NULL, *m1 = NULL;
 	XRRMonitorInfo		*rrm;
-	struct monitor 		*m = NULL;
+	char			*name = NULL;
     	int			 i, n = 0;
 	Window			 root = RootWindow(dpy, DefaultScreen(dpy));
 
-	rrm = XRRGetMonitors(dpy, root, false, &n);
+	rrm = XRRGetMonitors(dpy, root, true, &n);
 	if (n <= 0 && (!randr_initialised && monitor_get_count() == 0)) {
 		fvwm_debug(__func__, "get monitors failed\n");
 		exit(101);
 	}
 
+	/*
+	 * 1. Handle the case where we have no monitors at all.  This is going
+	 * to happen at init time.  In such cases, we can say that the
+	 * monitor_q is empty.
+	 *
+	 * To handle this case, we create all entries, marking them as new.
+	 */
+	if (!randr_initialised && monitor_get_count() == 0) {
+		fvwm_debug(__func__, "Case 1: Add new monitors");
+		for (i = 0; i < n; i++) {
+			monitor_add(&rrm[i]);
+		}
+		goto out;
+	}
+
+	/*
+	 * 2. Handle the case where an event has occurred for a monitor.  It
+	 * could be that:
+	 *   - It's a new monitor.
+	 *   - It's an existing monitor (position changed)
+	 *   - It's an existing monitor which has been toggled on or off.
+	 *
+	 *   In such cases, we must detect if the monitor exists and what
+	 *   state it is in.
+	 */
 	for (i = 0; i < n; i++) {
-
-		char *name = XGetAtomName(dpy, rrm[i].name);
-
-		if (name == NULL) {
-			fprintf(
-				stderr,
-				"%s: couldn't detect monitor with empty name\n",
-				__func__);
-			exit (101);
+		if ((name = XGetAtomName(dpy, rrm[i].name)) == NULL)
+			continue;
+		if ((m = monitor_by_name(name)) == NULL) {
+			/* Case 2.1 -- new monitor. */
+			fvwm_debug(__func__, "Case 2.1: new monitor");
+			monitor_add(&rrm[i]);
 		}
 
-		if (((m = monitor_by_name(name)) == NULL) ||
-		    (m != NULL && strcmp(m->si->name, name) != 0)) {
-			m = monitor_new();
-			m->flags |= MONITOR_NEW;
-			m->si = screen_info_new();
-			m->si->name = strdup(name);
-			memset(&m->virtual_scr, 0, sizeof(m->virtual_scr));
-
-			TAILQ_INSERT_TAIL(&screen_info_q, m->si, entry);
-			TAILQ_INSERT_TAIL(&monitor_q, m, entry);
-
-			goto set_coords;
+		if (monitor_mark_changed(m, &rrm[i])) {
+			fvwm_debug(__func__, "Case 2.2: %s changed", m->si->name);
 		}
-
-		if ((strcmp(m->si->name, name) == 0) &&
-		    (m->si->x != rrm[i].x || m->si->y != rrm[i].y ||
-		    m->si->w != rrm[i].width || m->si->h != rrm[i].height)) {
-			if (m->flags & MONITOR_ENABLED)
-				m->flags |= MONITOR_CHANGED;
-		}
-
-set_coords:
-		m->si->x = rrm[i].x;
-		m->si->y = rrm[i].y;
-		m->si->w = rrm[i].width;
-		m->si->h = rrm[i].height;
-		m->si->rr_output = *rrm[i].outputs;
-		if (rrm[i].primary > 0)
-			m->flags |= MONITOR_PRIMARY;
-		else
-			m->flags &= ~MONITOR_PRIMARY;
+		monitor_mark_inlist(name);
 
 		XFree(name);
 	}
 
-	monitor_scan_edges(m);
+out:
+	RB_FOREACH_SAFE(m, monitors, &monitor_q, m1) {
+		if (m->flags & MONITOR_CHANGED) {
+			m->flags &= ~MONITOR_DISABLED;
+			fvwm_debug(__func__, "REINSERTING: %s\n", m->si->name);
+			RB_REMOVE(monitors, &monitor_q, m);
+			RB_INSERT(monitors, &monitor_q, m);
+		}
+
+		/* Check for monitor connection status -- whether a monitor is
+		 * active or not.  Clearing the MONITOR_FOUND flag is
+		 * important here so that the monitor is reconsidered again.
+		 */
+		if (!(m->flags & (MONITOR_FOUND|MONITOR_NEW))) {
+			m->flags |= MONITOR_DISABLED;
+			m->emit |= MONITOR_DISABLED;
+		} else if ((m->flags & (MONITOR_FOUND|MONITOR_DISABLED)) ==
+			(MONITOR_FOUND|MONITOR_DISABLED)) {
+			m->flags &= ~MONITOR_DISABLED;
+		} else {
+			m->flags |= MONITOR_ENABLED;
+		}
+		m->flags &= ~MONITOR_FOUND;
+	}
+	/* Now that all monitors have been inserted, assign them a number from
+	 * 0 -> n so that they can be referenced in order.
+	 */
+	monitor_assign_number();
 	monitor_check_primary();
 	XRRFreeMonitors(rrm);
 }
@@ -504,12 +643,15 @@ void FScreenInit(Display *dpy)
 	is_randr_present = false;
 	randr_initialised = false;
 
-	if (TAILQ_EMPTY(&monitor_q))
-		TAILQ_INIT(&monitor_q);
+	if (RB_EMPTY(&monitor_q))
+		RB_INIT(&monitor_q);
 
 	if (TAILQ_EMPTY(&screen_info_q))
 		TAILQ_INIT(&screen_info_q);
 
+	if (TAILQ_EMPTY(&monitorsold_q))
+		TAILQ_INIT(&monitorsold_q);
+
 	if (!XRRQueryExtension(dpy, &randr_event, &err_base) ||
 	    !XRRQueryVersion (dpy, &major, &minor)) {
 		fvwm_debug(__func__, "RandR not present");
@@ -543,17 +685,8 @@ void FScreenInit(Display *dpy)
 	randr_initialised = true;
 	is_tracking_shared = false;
 
-	TAILQ_FOREACH(m, &monitor_q, entry) {
-		m->Desktops = fxcalloc(1, sizeof *m->Desktops);
-		m->Desktops->name = NULL;
-		m->Desktops->next = NULL;
-		m->Desktops->desk = 0;
-		m->flags |= (MONITOR_NEW|MONITOR_ENABLED);
-		m->is_prev = false;
+	RB_FOREACH(m, monitors, &monitor_q)
 		monitor_scan_edges(m);
-	}
-
-	monitor_check_primary();
 
 	return;
 
@@ -562,6 +695,22 @@ randr_fail:
 	exit(101);
 }
 
+void
+monitor_refresh_module(Display *dpy)
+{
+	struct monitor	*m = NULL;
+
+	RB_FOREACH(m, monitors, &monitor_q)
+		m->flags &= ~MONITOR_NEW;
+
+	scan_screens(dpy);
+
+	RB_FOREACH(m, monitors, &monitor_q) {
+		if (m->flags & MONITOR_NEW)
+			fprintf(stderr, "MON: %s is NEW...\n", m->si->name);
+	}
+}
+
 void
 monitor_dump_state(struct monitor *m)
 {
@@ -571,7 +720,7 @@ monitor_dump_state(struct monitor *m)
 
 	fvwm_debug(__func__, "Monitor Debug\n");
 	fvwm_debug(__func__, "\tnumber of outputs: %d\n", monitor_get_count());
-	TAILQ_FOREACH(m2, &monitor_q, entry) {
+	RB_FOREACH(m2, monitors, &monitor_q) {
 		if (m2 == NULL) {
 			fvwm_debug(__func__,
 				   "monitor in list is NULL.  Bug!\n");
@@ -581,6 +730,7 @@ monitor_dump_state(struct monitor *m)
 			continue;
 		fvwm_debug(
 			__func__,
+			"\tNumber:\t%d\n"
 			"\tName:\t%s\n"
 			"\tDisabled:\t%s\n"
 			"\tIs Primary:\t%s\n"
@@ -594,8 +744,14 @@ monitor_dump_state(struct monitor *m)
 			"\t\tCurrentDesk: %d\n"
 			"\t\tCurrentPage: {x: %d, y: %d}\n"
 			"\t\tMyDisplayWidth: %d, MyDisplayHeight: %d\n\t}\n"
+			"\tEWMH: {\n"
+			"\t\tBaseStrut Top:    %d\n"
+			"\t\tBaseStrut Bottom: %d\n"
+			"\t\tBaseStrut Left:   %d\n"
+			"\t\tBaseStrut Right:  %d\n\t}\n"
 			"\tDesktops:\t%s\n"
 			"\tFlags:%s\n\n",
+			m2->number,
 			m2->si->name,
 			(m2->flags & MONITOR_DISABLED) ? "true" : "false",
 			(m2->flags & MONITOR_PRIMARY) ? "yes" : "no",
@@ -612,6 +768,10 @@ monitor_dump_state(struct monitor *m)
 			(int)(m2->virtual_scr.Vy / monitor_get_all_heights()),
 			monitor_get_all_widths(),
 			monitor_get_all_heights(),
+			m2->ewmhc.BaseStrut.top,
+			m2->ewmhc.BaseStrut.bottom,
+			m2->ewmhc.BaseStrut.left,
+			m2->ewmhc.BaseStrut.right,
 			m2->Desktops ? "yes" : "no",
 			monitor_mode == MONITOR_TRACKING_G ? "global" :
 			monitor_mode == MONITOR_TRACKING_M ? "per-monitor" :
@@ -626,7 +786,10 @@ monitor_get_count(void)
 	struct monitor	*m = NULL;
 	int		 c = 0;
 
-	TAILQ_FOREACH(m, &monitor_q, entry) {
+	if (RB_EMPTY(&monitor_q))
+		return c;
+
+	RB_FOREACH(m, monitors, &monitor_q) {
 		if (m->flags & MONITOR_DISABLED)
 			continue;
 		c++;
@@ -638,19 +801,17 @@ struct monitor *
 FindScreenOfXY(int x, int y)
 {
 	struct monitor	*m;
-	int		 xa, ya;
-	int		 all_widths, all_heights;
+	int all_widths = monitor_get_all_widths();
+	int all_heights = monitor_get_all_heights();
 
-	all_widths =  monitor_get_all_widths();
-	all_heights = monitor_get_all_heights();
+	x %= all_widths;
+	y %= all_heights;
+	if (x < 0)
+		x += all_widths;
+	if (y < 0)
+		y += all_heights;
 
-	xa = abs(x);
-	ya = abs(y);
-
-	xa %= all_widths;
-	ya %= all_heights;
-
-	TAILQ_FOREACH(m, &monitor_q, entry) {
+	RB_FOREACH(m, monitors, &monitor_q) {
 		/* If we have more than one screen configured, then don't match
 		 * on the global screen, as that's separate to XY positioning
 		 * which is only concerned with the *specific* screen.
@@ -658,14 +819,14 @@ FindScreenOfXY(int x, int y)
 		if (monitor_get_count() > 0 &&
 		    strcmp(m->si->name, GLOBAL_SCREEN_NAME) == 0)
 			continue;
-		if (xa >= m->si->x && xa < m->si->x + m->si->w &&
-		    ya >= m->si->y && ya < m->si->y + m->si->h)
+		if (x >= m->si->x && x < m->si->x + m->si->w &&
+		    y >= m->si->y && y < m->si->y + m->si->h)
 			return (m);
 	}
 
 	/* FIXME: this is a convenience, but could confuse callers. */
 	if (m == NULL)
-		return TAILQ_FIRST(&monitor_q);
+		return RB_MIN(monitors, &monitor_q);
 
 	return (NULL);
 }
@@ -678,15 +839,13 @@ FindScreen(fscreen_scr_arg *arg, fscreen_scr_t screen)
 	fscreen_scr_arg  tmp;
 
 	if (monitor_get_count() == 0) {
-		monitor_refresh_global();
-		return (monitor_global);
+		return (monitor_get_global());
 	}
 
 	switch (screen)
 	{
 	case FSCREEN_GLOBAL:
-		monitor_refresh_global();
-		m = monitor_global;
+		m = monitor_get_global();
 		break;
 	case FSCREEN_PRIMARY:
 		m = monitor_by_primary();
@@ -769,8 +928,7 @@ Bool FScreenGetScrRect(fscreen_scr_arg *arg, fscreen_scr_t screen,
 	struct monitor	*m = FindScreen(arg, screen);
 	if (m == NULL) {
 		fvwm_debug(__func__, "m is NULL, using global screen\n");
-		monitor_refresh_global();
-		m = monitor_global;
+		m = monitor_get_global();
 	}
 
 	if (x)
@@ -986,7 +1144,7 @@ int FScreenParseGeometry(
 	char *parsestring, int *x_return, int *y_return,
 	unsigned int *width_return, unsigned int *height_return)
 {
-	struct monitor	*m = monitor_get_current();
+	struct monitor	*m = NULL;
 	int		 rc, x, y, w, h;
 
 	x = 0;
@@ -1001,11 +1159,16 @@ int FScreenParseGeometry(
 			parsestring, x_return, y_return, width_return,
 			height_return, &scr);
 
-		m = monitor_resolve_name(scr);
-		if (m == NULL)
-		{
-			/* fall back to current screen */
-			m = monitor_get_current();
+		if ((m = monitor_resolve_name(scr)) == NULL) {
+			/* If the monitor we tried to look up is NULL, it will
+			 * be because there was no geometry string specifying
+			 * the monitor name.
+			 *
+			 * In such cases, we should assume that the user has
+			 * requested the geometry to be relevant to the global
+			 * screen.
+			 */
+			m = monitor_get_global();
 		}
 		x = m->si->x;
 		y = m->si->y;
