@@ -6,6 +6,10 @@
 #include "fvwm_x11.h"
 #include "fvwmrect.h"
 
+#include "queue.h"
+#include "tree.h"
+
+#include <X11/extensions/Xrandr.h>
 #include <stdbool.h>
 
 typedef struct
@@ -61,7 +65,7 @@ struct screen_info {
 };
 TAILQ_HEAD(screen_infos, screen_info);
 
-extern struct screen_infos	 screen_info_q;
+extern struct screen_infos	 screen_info_q, screen_info_q_temp;
 
 struct screen_info	*screen_info_new(void);
 struct screen_info	*screen_info_by_name(const char *);
@@ -70,8 +74,8 @@ struct screen_info	*screen_info_by_name(const char *);
 #define MONITOR_DISABLED 0x2
 #define MONITOR_ENABLED 0x4
 #define MONITOR_PRIMARY 0x8
-#define MONITOR_CHANGED 0x10
-#define MONITOR_ALL (MONITOR_DISABLED|MONITOR_ENABLED|MONITOR_CHANGED)
+#define MONITOR_CHANGED 0x100
+#define MONITOR_FOUND 0x200
 
 #define MONITOR_OUTSIDE_EDGE 0
 #define MONITOR_INSIDE_EDGE 1
@@ -80,8 +84,9 @@ struct monitor {
 	struct screen_info	*si;
 	int			 flags;
 	int			 emit;
+	int			 number;
 	int			 dx, dy;
-	bool			 is_prev;
+	bool			 is_prev, was_primary;
 
 	/* info for some desktops; the first entries should be generic info
          * correct for any desktop not in the list
@@ -117,6 +122,9 @@ struct monitor {
                 int Vx;
                 int Vy;
 
+		int edge_thickness;
+		int last_edge_thickness;
+
                 int EdgeScrollX;
                 int EdgeScrollY;
 
@@ -135,17 +143,27 @@ struct monitor {
 	PanFrame PanFrameLeft;
 	PanFrame PanFrameRight;
 	PanFrame PanFrameBottom;
+	bool pan_frames_mapped;
 
-	TAILQ_ENTRY(monitor) entry;
+	RB_ENTRY(monitor) entry;
+	TAILQ_ENTRY(monitor) oentry;
 };
-TAILQ_HEAD(monitors, monitor);
+RB_HEAD(monitors, monitor);
+TAILQ_HEAD(monitorsold, monitor);
 
-extern struct monitors		monitor_q;
+extern struct monitors  monitors;
+extern struct monitors	monitor_q;
+extern struct monitorsold  monitorsold_q;
+int			monitor_compare(struct monitor *, struct monitor *);
+RB_PROTOTYPE(monitors, monitor, entry, monitor_compare);
 
+int		 changed_monitor_count(void);
 struct monitor	*monitor_resolve_name(const char *);
 struct monitor	*monitor_by_xy(int, int);
 struct monitor  *monitor_by_output(int);
+struct monitor	*monitor_by_number(int);
 struct monitor  *monitor_by_primary(void);
+struct monitor  *monitor_by_last_primary(void);
 struct monitor  *monitor_get_current(void);
 struct monitor  *monitor_get_prev(void);
 struct monitor  *monitor_get_global(void);
@@ -154,8 +172,8 @@ void		 monitor_dump_state(struct monitor *);
 void		 monitor_output_change(Display *, XRRScreenChangeNotifyEvent *);
 int		 monitor_get_all_widths(void);
 int		 monitor_get_all_heights(void);
-void		 monitor_add_new(void);
 void		 monitor_assign_virtual(struct monitor *);
+void		 monitor_refresh_module(Display *);
 void		 checkPanFrames(struct monitor *);
 
 #define FSCREEN_MANGLE_USPOS_HINTS_MAGIC ((short)-32109)
@@ -204,4 +222,4 @@ int  FScreenGetGeometry(
 void FScreenMangleScreenIntoUSPosHints(fscreen_scr_t screen, XSizeHints *hints);
 int FScreenFetchMangledScreenFromUSPosHints(XSizeHints *hints);
 
-#endif /* FVWMLIB_FSCRREN_H */
+#endif /* FVWMLIB_FSCRREN_H */
\ No newline at end of file
