@@ -1461,11 +1461,8 @@ FlocaleFont *FlocaleLoadFont(Display *dpy, char *fontname, char *module)
 				&shadow_dir, fontname, module);
 			break;
 		case 1: /* encoding= */
-			if (encoding != NULL)
-			{
-				free(encoding);
-				encoding = NULL;
-			}
+			free(encoding);
+			encoding = NULL;
 			if (opt_str && *opt_str)
 			{
 				CopyString(&encoding, opt_str);
@@ -1474,8 +1471,7 @@ FlocaleFont *FlocaleLoadFont(Display *dpy, char *fontname, char *module)
 		default:
 			break;
 		}
-		if (opt_str != NULL)
-			free(opt_str);
+		free(opt_str);
 	}
 	if (str && *str)
 	{
@@ -1647,10 +1643,8 @@ void FlocaleUnloadFont(Display *dpy, FlocaleFont *flf)
 	}
 	if (flf->flags.must_free_fc)
 	{
-		if (flf->fc->x)
-			free(flf->fc->x);
-		if (flf->fc->bidi)
-			free(flf->fc->bidi);
+		free(flf->fc->x);
+		free(flf->fc->bidi);
 		if (flf->fc->locale != NULL)
 		{
 			while (FLC_GET_LOCALE_CHARSET(flf->fc,i) != NULL)
@@ -2089,24 +2083,16 @@ void FlocaleDrawString(
 
 	if (do_free)
 	{
-		if (fws->e_str != NULL)
-		{
-			free(fws->e_str);
-			fws->e_str = NULL;
-		}
-	}
-
-	if (fws->str2b != NULL)
-	{
-		free(fws->str2b);
-		fws->str2b = NULL;
+		free(fws->e_str);
+		fws->e_str = NULL;
 	}
+	free(fws->str2b);
+	fws->str2b = NULL;
 
 	if(comb_chars != NULL)
 	{
 	        free(comb_chars);
-		if(pixel_pos)
-			free(pixel_pos);
+		free(pixel_pos);
 	}
 
 	return;
@@ -2160,12 +2146,8 @@ void FlocaleDrawUnderline(
 		free(fws->e_str);
 		fws->e_str = NULL;
 	}
-
-	if(fws->str2b != NULL)
-	{
-		free(fws->str2b);
-		fws->str2b = NULL;
-	}
+	free(fws->str2b);
+	fws->str2b = NULL;
 	free(l_to_v);
 
 	return;
