@@ -737,22 +737,20 @@ void X_init_manager (int man_id)
 	man->geometry.boxheight = height;
     }
   }
-  FScreenGetScrRect(NULL, FSCREEN_PRIMARY, &man->managed_g.x, &man->managed_g.y,
-		    &man->managed_g.width, &man->managed_g.height);
+
+  /* Populate the managed instance of this FvwmIconMan with the position of
+   * the global screen.  This is potentially overriden if the user supplies
+   * their own ManagerGeometry string.
+   */
+  FScreenGetScrRect(NULL, FSCREEN_GLOBAL, &man->managed_g.x, &man->managed_g.y,
+		  &man->managed_g.width, &man->managed_g.height);
   man->geometry.x = man->managed_g.x;
   man->geometry.y = man->managed_g.y;
+
   if (man->geometry_str) {
-    fscreen_scr_arg arg;
-    arg.mouse_ev = NULL;
-    arg.name = NULL;
-
-    geometry_mask = FScreenParseGeometryWithScreen(
+    geometry_mask = FScreenParseGeometry(
       man->geometry_str, &man->geometry.x, &man->geometry.y,
-      &man->geometry.cols, &man->geometry.rows, NULL);
-
-    FScreenGetScrRect(
-      &arg, FSCREEN_BY_NAME, &man->managed_g.x, &man->managed_g.y,
-      &man->managed_g.width, &man->managed_g.height);
+      &man->geometry.cols, &man->geometry.rows);
 
     if (geometry_mask & XValue)
       man->geometry.x += man->managed_g.x;
@@ -802,14 +800,9 @@ void X_init_manager (int man_id)
     unsigned int ujunk;
     fscreen_scr_arg fscr;
 
-    if (FQueryPointer(
+    FQueryPointer(
 	  theDisplay, theRoot, &dummyroot, &dummychild, &man->geometry.x,
-	  &man->geometry.y, &junk, &junk, &ujunk) == False)
-    {
-      /* pointer is on a different screen - that's okay here */
-      man->geometry.y = 0;
-      man->geometry.x = 0;
-    }
+	  &man->geometry.y, &junk, &junk, &ujunk);
     fscr.xypos.x = man->geometry.x;
     fscr.xypos.y = man->geometry.y;
     FScreenGetScrRect(
