@@ -1,4 +1,3 @@
-/* -*-c-*- */
 /* This module, and the entire ModuleDebugger program, and the concept for
  * interfacing this module to the Window Manager, are all original work
  * by Robert Nation
@@ -24,6 +23,7 @@
 #include <signal.h>
 #include <fcntl.h>
 #include <sys/wait.h>
+#include "libs/FScreen.h"
 #include "libs/ftime.h"
 #include <ctype.h>
 
@@ -56,7 +56,7 @@ PagerStringList *NewPagerStringItem(PagerStringList *last, int desk);
 extern FlocaleFont *FwindowFont;
 extern Pixmap default_pixmap;
 
-struct fpmonitors		 fp_monitor_q;
+struct fpmonitors fp_monitor_q;
 
 /*
  *
@@ -112,13 +112,16 @@ char *BalloonFont = NULL;
 char *BalloonBorderColor = NULL;
 char *BalloonFormatString = NULL;
 char *monitor_to_track = NULL;
+char *preferred_monitor = NULL;
 int BalloonBorderWidth = 1;
 int BalloonYOffset = 3;
 Window BalloonView = None;
 
 rectangle pwindow = {0, 0, 0, 0};
 rectangle icon = {-10000, -10000, 0, 0};
-int usposition = 0,uselabel = 1;
+int usposition = 0;
+Bool use_desk_label = True;
+Bool use_monitor_label = False;
 int xneg = 0, yneg = 0;
 int icon_xneg = 0, icon_yneg = 0;
 extern DeskInfo *Desks;
@@ -146,67 +149,88 @@ Bool is_transient = False;
 Bool do_ignore_next_button_release = False;
 Bool error_occured = False;
 Bool Swallowed = False;
+Bool fp_new_block = False;
 
-static void extract_monitor_config(struct fpmonitor *, char *);
 static void SetDeskLabel(struct fpmonitor *, int desk, const char *label);
 static RETSIGTYPE TerminateHandler(int);
 void ExitPager(void);
 void list_monitor_focus(unsigned long *);
+struct fpmonitor *fpmonitor_new(struct monitor *);
+void fpmonitor_disable(struct fpmonitor *);
 
 struct fpmonitor *
-fpmonitor_get_current(void)
+fpmonitor_new(struct monitor *m)
 {
-	struct fpmonitor *m;
+	struct fpmonitor	*fp;
 
-	TAILQ_FOREACH(m, &fp_monitor_q, entry) {
-		if (m->is_current)
-			return (m);
+	fp = fxcalloc(1, sizeof(*fp));
+	fp->CPagerWin = fxcalloc(1, ndesks * sizeof(*fp->CPagerWin));
+	fp->m = m;
+	fp->disabled = false;
+	TAILQ_INSERT_TAIL(&fp_monitor_q, fp, entry);
+
+	return (fp);
+}
+
+void fpmonitor_disable(struct fpmonitor *fp)
+{
+	fp->disabled = true;
+	for (int i = 0; i < ndesks; i++) {
+		XMoveWindow(dpy, fp->CPagerWin[i], -32768,-32768);
+	}
+
+	if (monitor_to_track != NULL &&
+		strcmp(monitor_to_track, fp->m->si->name) == 0)
+	{
+		free(monitor_to_track);
+		struct fpmonitor *tm = fpmonitor_this(NULL);
+		monitor_to_track = fxstrdup(tm->m->si->name);
 	}
-	return (NULL);
 }
 
 struct fpmonitor *
 fpmonitor_by_name(const char *name)
 {
-	struct fpmonitor	*m;
+	struct fpmonitor *fm;
 
-	TAILQ_FOREACH(m, &fp_monitor_q, entry) {
-		if (strcmp(m->name, name) == 0)
-			return (m);
+	if (name == NULL)
+		return (NULL);
+
+	TAILQ_FOREACH(fm, &fp_monitor_q, entry) {
+		if (fm->m != NULL && (strcmp(name, fm->m->si->name) == 0))
+			return (fm);
 	}
 	return (NULL);
 }
 
 struct fpmonitor *
-fpmonitor_by_output(int output)
+fpmonitor_this(struct monitor *m_find)
 {
-	struct fpmonitor	*m;
+	struct monitor *m = NULL;
+	struct fpmonitor *fp = NULL;
 
-	TAILQ_FOREACH(m, &fp_monitor_q, entry) {
-		if (m->output == output)
-			return (m);
+	if (m_find != NULL) {
+		/* We've been asked to find a specific monitor. */
+		m = m_find;
+		goto search;
 	}
-	return (NULL);
-}
-
-
-
-struct fpmonitor *
-fpmonitor_this(void)
-{
-	struct fpmonitor 	*m;
-
 	if (monitor_to_track == NULL)
-		return (fpmonitor_get_current());
+		m = monitor_get_current();
+	else
+		m = monitor_resolve_name(monitor_to_track);
 
-	TAILQ_FOREACH(m, &fp_monitor_q, entry) {
-		if (monitor_to_track != NULL &&
-		    strcmp(m->name, monitor_to_track) == 0) {
-			return (m);
-		}
+	if (m == NULL)
+		m = monitor_get_current();
+
+search:
+	if (m == NULL || m->flags & MONITOR_DISABLED)
+		return (NULL);
+	TAILQ_FOREACH(fp, &fp_monitor_q, entry) {
+		if (fp->m == m)
+			return (fp);
 	}
+	return (NULL);
 
-	return (fpmonitor_get_current());
 }
 
 /*
@@ -224,6 +248,7 @@ int main(int argc, char **argv)
   Window JunkRoot, JunkChild;
   int JunkX, JunkY;
   unsigned JunkMask;
+  struct monitor *mon;
 
   FlocaleInit(LC_CTYPE, "", "", "FvwmPager");
 
@@ -369,9 +394,10 @@ int main(int argc, char **argv)
       exit (1);
     }
 
-  x_fd = XConnectionNumber(dpy);
   flib_init_graphics(dpy);
 
+  x_fd = XConnectionNumber(dpy);
+
   Scr.screen = DefaultScreen(dpy);
   Scr.Root = RootWindow(dpy, Scr.screen);
   /* make a temp window for any pixmaps, deleted later */
@@ -402,17 +428,15 @@ int main(int argc, char **argv)
 		 MX_PROPERTY_CHANGE|
 		 MX_MONITOR_FOCUS);
 
+  RB_FOREACH(mon, monitors, &monitor_q)
+	(void)fpmonitor_new(mon);
+
   ParseOptions();
   if (is_transient)
   {
-    if (FQueryPointer(
+    FQueryPointer(
 	  dpy, Scr.Root, &JunkRoot, &JunkChild, &pwindow.x, &pwindow.y, &JunkX,
-	  &JunkY, &JunkMask) == False)
-    {
-      /* pointer is on a different screen */
-      pwindow.x = 0;
-      pwindow.y = 0;
-    }
+	  &JunkY, &JunkMask);
     usposition = 1;
     xneg = 0;
     yneg = 0;
@@ -448,6 +472,7 @@ int main(int argc, char **argv)
   SendInfo(fd,"Send_WindowList",0);
 
   /* open a pager window */
+  initialise_common_pager_fragments();
   initialize_pager();
 
   if (is_transient)
@@ -659,7 +684,7 @@ void handle_config_win_package(PagerWindow *t,
 	t->frame_y = cfgpacket->frame_y;
 	t->frame_width = cfgpacket->frame_width;
 	t->frame_height = cfgpacket->frame_height;
-	t->m = fpmonitor_by_output(cfgpacket->monitor_id);
+	t->m = monitor_by_output(cfgpacket->monitor_id);
 
 	t->desk = cfgpacket->desk;
 
@@ -717,18 +742,11 @@ void handle_config_win_package(PagerWindow *t,
 void list_add(unsigned long *body)
 {
 	PagerWindow *t,**prev;
-	int i=0;
 	struct ConfigWinPacket  *cfgpacket = (void *) body;
-	struct fpmonitor *newm;
 
 	t = Start;
 	prev = &Start;
 
-	newm = fpmonitor_by_output((int)cfgpacket->monitor_id);
-
-	if (newm == NULL)
-		newm = fpmonitor_get_current();
-
 	while(t != NULL)
 	{
 		if (t->w == cfgpacket->w)
@@ -738,7 +756,6 @@ void list_add(unsigned long *body)
 		}
 		prev = &(t->next);
 		t = t->next;
-		i++;
 	}
 	*prev = fxcalloc(1, sizeof(PagerWindow));
 	handle_config_win_package(*prev, cfgpacket);
@@ -759,19 +776,11 @@ void list_configure(unsigned long *body)
   Window target_w;
   struct ConfigWinPacket  *cfgpacket = (void *) body;
   Bool is_new_desk, is_new_monitor;
-  struct fpmonitor *newm;
+  struct monitor *newm;
 
   target_w = cfgpacket->w;
   t = Start;
-  newm = fpmonitor_by_output((int)cfgpacket->monitor_id);
-
-  if (newm == NULL) {
-	  fvwm_debug(__func__, "monitor was null with ID: %d\n",
-                     (int)cfgpacket->monitor_id);
-	  fvwm_debug(__func__, "using current montior\n");
-
-	  newm = fpmonitor_get_current();
-  }
+  newm = monitor_by_output((int)cfgpacket->monitor_id);
 
   while( (t != NULL) && (t->w != target_w))
   {
@@ -839,38 +848,17 @@ void list_destroy(unsigned long *body)
       XDestroyWindow(dpy,t->IconView);
       if(FocusWin == t)
 	FocusWin = NULL;
-      if(t->res_class != NULL)
-	free(t->res_class);
-      if(t->res_name != NULL)
-	free(t->res_name);
-      if(t->window_name != NULL)
-	free(t->window_name);
-      if(t->icon_name != NULL)
-	free(t->icon_name);
+      free(t->res_class);
+      free(t->res_name);
+      free(t->window_name);
+      free(t->icon_name);
       free(t);
     }
 }
 
 void list_monitor_focus(unsigned long *body)
 {
-	struct fpmonitor	*m, *mcur, *mthis;
-	char	*mon_name = (char *)(&body[3]);
-
-	mcur = fpmonitor_get_current();
-
-	if (mcur == NULL || mon_name == NULL)
-		return;
-
-	mthis = fpmonitor_by_name(mon_name);
-
-	if (mcur == mthis)
-		return;
-
-	mcur->is_current = 0;
-	TAILQ_FOREACH(m, &fp_monitor_q, entry) {
-		if (m == mthis)
-			m->is_current = 1;
-	}
+	return;
 }
 
 /*
@@ -929,35 +917,43 @@ void list_focus(unsigned long *body)
 void list_new_page(unsigned long *body)
 {
 	int mon_num = body[7];
-	struct fpmonitor *m;
+	struct monitor *m;
+	struct fpmonitor *fp;
 
-	m = fpmonitor_by_output(mon_num);
-	fvwm_debug(__func__, "%s: using monitor: %s\n", __func__, m->name);
-
-	if (monitor_to_track != NULL && strcmp(m->name, monitor_to_track) != 0)
+	m = monitor_by_output(mon_num);
+	/* Don't allow monitor_by_output to fallback to RB_MIN. */
+	if (m->si->rr_output != mon_num)
 		return;
 
-  m->virtual_scr.Vx = body[0];
-  m->virtual_scr.Vy = body[1];
-  if (m->virtual_scr.CurrentDesk != body[2])
-  {
-      /* first handle the new desk */
-      body[0] = body[2];
-      list_new_desk(body);
-  }
-  if (m->virtual_scr.VxPages != body[5] || m->virtual_scr.VyPages != body[6])
-  {
-    m->virtual_scr.VxPages = body[5];
-    m->virtual_scr.VyPages = body[6];
-    m->virtual_scr.VWidth = m->virtual_scr.VxPages * m->virtual_scr.MyDisplayWidth;
-    m->virtual_scr.VHeight = m->virtual_scr.VyPages * m->virtual_scr.MyDisplayHeight;
-    m->virtual_scr.VxMax = m->virtual_scr.VWidth - m->virtual_scr.MyDisplayWidth;
-    m->virtual_scr.VyMax = m->virtual_scr.VHeight - m->virtual_scr.MyDisplayHeight;
-    ReConfigure();
-  }
-  MovePage(False);
-  MoveStickyWindow(True, False);
-  Hilight(FocusWin,True);
+	if (monitor_to_track != NULL &&
+	    strcmp(m->si->name, monitor_to_track) != 0)
+		return;
+
+	if ((fp = fpmonitor_this(m)) == NULL)
+		return;
+
+	fp->virtual_scr.Vx = body[0];
+	fp->virtual_scr.Vy = body[1];
+	if (fp->m->virtual_scr.CurrentDesk != body[2]) {
+		/* first handle the new desk */
+		body[0] = body[2];
+		body[1] = mon_num;
+		list_new_desk(body);
+	}
+	if (fp->virtual_scr.VxPages != body[5] || fp->virtual_scr.VyPages != body[6])
+	{
+		fp->virtual_scr.VxPages = body[5];
+		fp->virtual_scr.VyPages = body[6];
+		fp->virtual_scr.VWidth = fp->virtual_scr.VxPages * fpmonitor_get_all_widths();
+		fp->virtual_scr.VHeight = fp->virtual_scr.VyPages * fpmonitor_get_all_heights();
+		fp->virtual_scr.VxMax = fp->virtual_scr.VWidth - fpmonitor_get_all_widths();
+		fp->virtual_scr.VyMax = fp->virtual_scr.VHeight - fpmonitor_get_all_heights();
+		ReConfigure();
+	}
+
+	MovePage(False);
+	MoveStickyWindow(True, False);
+	Hilight(FocusWin,True);
 }
 
 /*
@@ -972,39 +968,52 @@ void list_new_desk(unsigned long *body)
   int change_cs = -1;
   int change_ballooncs = -1;
   int change_highcs = -1;
-  struct fpmonitor *m, *mout;
+  int mon_num = body[1];
+  struct monitor *mout;
+  struct fpmonitor *fp;
 
-  m = fpmonitor_this();
-  mout = fpmonitor_by_output((int)body[1]);
+  mout = monitor_by_output(mon_num);
+  /* Don't allow monitor_by_output to fallback to RB_MIN. */
+  if (mout->si->rr_output != mon_num)
+	  return;
+
+  fp = fpmonitor_this(mout);
+
+  if (fp == NULL)
+	  return;
+
+  /* Best to update the desk, even if the monitor isn't being
+   * tracked, as this could change.
+   */
+  oldDesk = fp->m->virtual_scr.CurrentDesk;
+  fp->m->virtual_scr.CurrentDesk = (long)body[0];
 
   /* If the monitor for which the event was sent, does not match the monitor
    * itself, then don't change the FvwmPager's desk.  Only do this if we're
    * tracking a specific monitor though.
    */
-  if (monitor_to_track != NULL && m != mout)
+  if (monitor_to_track != NULL && fp->m != mout)
 	  return;
 
-  oldDesk = m->virtual_scr.CurrentDesk;
-  m->virtual_scr.CurrentDesk = (long)body[0];
-  if (fAlwaysCurrentDesk && oldDesk != m->virtual_scr.CurrentDesk)
+  if (monitor_mode == MONITOR_TRACKING_G)
+    monitor_assign_virtual(fp->m);
+
+  if (fAlwaysCurrentDesk && oldDesk != fp->m->virtual_scr.CurrentDesk)
   {
     PagerWindow *t;
     PagerStringList *item;
     char line[100];
 
-    desk1 = m->virtual_scr.CurrentDesk;
-    desk2 = m->virtual_scr.CurrentDesk;
+    desk1 = fp->m->virtual_scr.CurrentDesk;
+    desk2 = fp->m->virtual_scr.CurrentDesk;
     for (t = Start; t != NULL; t = t->next)
     {
-      if (t->desk == oldDesk || t->desk == m->virtual_scr.CurrentDesk)
+      if (t->desk == oldDesk || t->desk == fp->m->virtual_scr.CurrentDesk)
 	ChangeDeskForWindow(t, t->desk);
     }
-    item = FindDeskStrings(m->virtual_scr.CurrentDesk);
-    if (Desks[0].label != NULL)
-    {
-      free(Desks[0].label);
-      Desks[0].label = NULL;
-    }
+    item = FindDeskStrings(fp->m->virtual_scr.CurrentDesk);
+    free(Desks[0].label);
+    Desks[0].label = NULL;
     if (item->next != NULL && item->next->label != NULL)
     {
       CopyString(&Desks[0].label, item->next->label);
@@ -1022,12 +1031,8 @@ void list_new_desk(unsigned long *body)
       PDestroyFvwmPicture(dpy, Desks[0].bgPixmap);
       Desks[0].bgPixmap = NULL;
     }
-
-    if (Desks[0].Dcolor != NULL)
-    {
-      free (Desks[0].Dcolor);
-      Desks[0].Dcolor = NULL;
-    }
+    free (Desks[0].Dcolor);
+    Desks[0].Dcolor = NULL;
 
     if (item->next != NULL)
     {
@@ -1110,14 +1115,14 @@ void list_new_desk(unsigned long *body)
     char *name;
     char line[100];
 
-    i = m->virtual_scr.CurrentDesk - desk1;
+    i = fp->m->virtual_scr.CurrentDesk - desk1;
     if (i >= 0 && i < ndesks && Desks[i].label != NULL)
     {
       name = Desks[i].label;
     }
     else
     {
-      snprintf(line, sizeof(line), "Desk %d", m->virtual_scr.CurrentDesk);
+      snprintf(line, sizeof(line), "Desk %d", fp->m->virtual_scr.CurrentDesk);
       name = &(line[0]);
     }
     XStoreName(dpy, Scr.Pager_w, name);
@@ -1126,7 +1131,7 @@ void list_new_desk(unsigned long *body)
 
   MovePage(True);
   DrawGrid(oldDesk - desk1, 1, None, NULL);
-  DrawGrid(m->virtual_scr.CurrentDesk - desk1, 1, None, NULL);
+  DrawGrid(fp->m->virtual_scr.CurrentDesk - desk1, 1, None, NULL);
   MoveStickyWindow(False, True);
 /*
   Hilight(FocusWin,False);
@@ -1170,6 +1175,7 @@ void list_lower(unsigned long *body)
 {
   PagerWindow *t;
   Window target_w;
+  struct fpmonitor *m;
 
   target_w = body[0];
   t = Start;
@@ -1181,8 +1187,11 @@ void list_lower(unsigned long *body)
     {
       if(t->PagerView != None)
 	XLowerWindow(dpy,t->PagerView);
-      if (HilightDesks && (t->desk - desk1>=0) && (t->desk - desk1<ndesks))
-	XLowerWindow(dpy,Desks[t->desk - desk1].CPagerWin);
+      if (HilightDesks && (t->desk - desk1>=0) && (t->desk - desk1<ndesks)) {
+	TAILQ_FOREACH(m, &fp_monitor_q, entry) {
+		XLowerWindow(dpy, m->CPagerWin[t->desk - desk1]);
+	}
+      }
       XLowerWindow(dpy,t->IconView);
     }
 }
@@ -1291,17 +1300,19 @@ void list_window_name(unsigned long *body,unsigned long type)
 {
   PagerWindow *t;
   Window target_w;
-  struct fpmonitor *m = fpmonitor_this();
+  struct fpmonitor *fp = fpmonitor_this(NULL);
 
+  if (fp == NULL)
+    return;
 
-  if (monitor_to_track != NULL && strcmp(m->name, monitor_to_track) != 0)
+  if (monitor_to_track != NULL && strcmp(fp->m->si->name, monitor_to_track) != 0)
 	  return;
 
   target_w = body[0];
   t = Start;
   while((t!= NULL)&&(t->w != target_w))
     {
-	    if (m != NULL && t->m != m) {
+	    if (fp->m != NULL && t->m != fp->m) {
 		    t = t->next;
 		    continue;
 	    }
@@ -1311,18 +1322,15 @@ void list_window_name(unsigned long *body,unsigned long type)
     {
       switch (type) {
       case M_RES_CLASS:
-	if(t->res_class != NULL)
-	  free(t->res_class);
+	free(t->res_class);
 	CopyString(&t->res_class,(char *)(&body[3]));
 	break;
       case M_RES_NAME:
-	if(t->res_name != NULL)
-	  free(t->res_name);
+	free(t->res_name);
 	CopyString(&t->res_name,(char *)(&body[3]));
 	break;
       case M_VISIBLE_NAME:
-	if(t->window_name != NULL)
-	  free(t->window_name);
+	free(t->window_name);
 	CopyString(&t->window_name,(char *)(&body[3]));
 	break;
       }
@@ -1370,8 +1378,7 @@ void list_icon_name(unsigned long *body)
     }
   if(t!= NULL)
     {
-      if(t->icon_name != NULL)
-	free(t->icon_name);
+      free(t->icon_name);
       CopyString(&t->icon_name,(char *)(&body[3]));
       /* repaint by clearing window */
       if ((FwindowFont != NULL) && (t->icon_name != NULL)
@@ -1499,33 +1506,9 @@ void list_end(void)
     XFree((char *)children);
 }
 
-void extract_monitor_config(struct fpmonitor *m, char *tline)
-{
-    int  output, mdw, mdh, vx, vy, vxmax, vymax, iscur;
-    int  x, y, w, h;
-
-    sscanf(tline, "%d %d %d %d %d %d %d %d %d %d %d %d",
-        &output, &iscur, &mdw, &mdh, &vx, &vy, &vxmax, &vymax,
-        &x, &y, &w, &h);
-    m->x = x;
-    m->y = y;
-    m->w = w;
-    m->h = h;
-    m->output = output;
-    m->is_current = iscur;
-    m->virtual_scr.MyDisplayWidth = mdw;
-    m->virtual_scr.MyDisplayHeight = mdh;
-    m->virtual_scr.Vx = vx;
-    m->virtual_scr.Vy = vy;
-    m->virtual_scr.VxMax = vxmax;
-    m->virtual_scr.VyMax = vymax;
-
-}
-
-
 void list_config_info(unsigned long *body)
 {
-	struct fpmonitor *m, *m2;
+	struct fpmonitor *m;
 	char *tline, *token;
 	int color;
 
@@ -1551,7 +1534,7 @@ void list_config_info(unsigned long *body)
 		if (fAlwaysCurrentDesk)
 		{
 			TAILQ_FOREACH(m, &fp_monitor_q, entry) {
-				if (m->virtual_scr.CurrentDesk == val)
+				if (m->m->virtual_scr.CurrentDesk == val)
 					val = 0;
 			}
 		}
@@ -1561,27 +1544,83 @@ void list_config_info(unsigned long *body)
 		}
 		DrawGrid(val, True, None, NULL);
 	} else if (StrEquals(token, "Monitor")) {
-		char	*mname;
-		int		 updated = 0;
+		int		  dx, dy, Vx, Vy, VxMax, VyMax, CurrentDesk;
+		int		  scr_width, scr_height;
+		int		  flags;
+		char		 *mname;
+		struct monitor	 *tm;
+		struct fpmonitor *fp;
 
 		tline = GetNextToken(tline, &mname);
 
-		TAILQ_FOREACH(m2, &fp_monitor_q, entry) {
-			updated = 0;
-			if (strcmp(m2->name, mname) == 0) {
-				extract_monitor_config(m2,tline);
-				updated = 1;
-			}
-		}
+		sscanf(tline, "%d %d %d %d %d %d %d %d %d %d", &flags,
+		    &dx, &dy, &Vx, &Vy, &VxMax, &VyMax, &CurrentDesk,
+		    &scr_width, &scr_height);
 
-		if (updated)
+		monitor_refresh_module(dpy);
+
+		tm = monitor_resolve_name(mname);
+		if (tm == NULL)
 			return;
 
-		m = fxcalloc(1, sizeof(*m));
+		if (flags & MONITOR_DISABLED) {
+			fp = fpmonitor_by_name(mname);
+			if (fp != NULL) {
+				bool disabled = fp->disabled;
+				fpmonitor_disable(fp);
+				if (!disabled)
+					ReConfigure();
+			}
+			return;
+		}
 
-		m->name = fxstrdup(mname);
-		extract_monitor_config(m,tline);
-		TAILQ_INSERT_TAIL(&fp_monitor_q, m, entry);
+		if ((fp = fpmonitor_this(tm)) == NULL) {
+			if (fp_new_block)
+				return;
+			fp_new_block = True;
+			fp = fpmonitor_new(tm);
+			fp->m->flags |= MONITOR_NEW;
+		}
+
+		/* This ensures that if monitor_to_track gets disconnected
+		 * then reconnected, the pager can resume tracking it.
+		 */
+		if (preferred_monitor != NULL &&
+			strcmp(fp->m->si->name, preferred_monitor) == 0 &&
+			strcmp(preferred_monitor, monitor_to_track) != 0)
+		{
+			if (monitor_to_track != NULL)
+				free(monitor_to_track);
+			monitor_to_track = fxstrdup(preferred_monitor);
+		}
+		fp->disabled = false;
+		fp->scr_width = scr_width;
+		fp->scr_height = scr_height;
+		fp->m->dx = dx;
+		fp->m->dy = dy;
+		fp->m->virtual_scr.Vx = fp->virtual_scr.Vx = Vx;
+		fp->m->virtual_scr.Vy = fp->virtual_scr.Vy = Vy;
+		fp->m->virtual_scr.VxMax = fp->virtual_scr.VxMax = VxMax;
+		fp->m->virtual_scr.VyMax = fp->virtual_scr.VyMax = VyMax;
+		fp->m->virtual_scr.CurrentDesk = CurrentDesk;
+
+		fp->virtual_scr.VxMax = dx * fpmonitor_get_all_widths() - fpmonitor_get_all_widths();
+		fp->virtual_scr.VyMax = dy * fpmonitor_get_all_heights() - fpmonitor_get_all_heights();
+		if (fp->virtual_scr.VxMax < 0)
+			fp->virtual_scr.VxMax = 0;
+		if (fp->virtual_scr.VyMax < 0)
+			fp->virtual_scr.VyMax = 0;
+		fp->virtual_scr.VWidth = fp->virtual_scr.VxMax + fpmonitor_get_all_widths();
+		fp->virtual_scr.VHeight = fp->virtual_scr.VyMax + fpmonitor_get_all_heights();
+		fp->virtual_scr.VxPages = fp->virtual_scr.VWidth / fpmonitor_get_all_widths();
+		fp->virtual_scr.VyPages = fp->virtual_scr.VHeight / fpmonitor_get_all_heights();
+
+		if (fp->m != NULL && fp->m->flags & MONITOR_NEW) {
+			fp->m->flags &= ~MONITOR_NEW;
+			initialize_fpmonitor_windows(fp);
+			fp_new_block = False;
+		}
+		ReConfigure();
 	} else if (StrEquals(token, "DesktopSize")) {
 		int dx, dy;
 		struct fpmonitor *m;
@@ -1589,17 +1628,24 @@ void list_config_info(unsigned long *body)
 		sscanf(tline, "%d %d", &dx, &dy);
 
 		TAILQ_FOREACH(m, &fp_monitor_q, entry) {
-			m->virtual_scr.VxMax = dx * m->virtual_scr.MyDisplayWidth - m->virtual_scr.MyDisplayWidth;
-			m->virtual_scr.VyMax = dy * m->virtual_scr.MyDisplayHeight - m->virtual_scr.MyDisplayHeight;
+			m->virtual_scr.VxMax = dx * fpmonitor_get_all_widths() - fpmonitor_get_all_widths();
+			m->virtual_scr.VyMax = dy * fpmonitor_get_all_heights() - fpmonitor_get_all_heights();
 			if (m->virtual_scr.VxMax < 0)
 				m->virtual_scr.VxMax = 0;
 			if (m->virtual_scr.VyMax < 0)
 				m->virtual_scr.VyMax = 0;
-			m->virtual_scr.VWidth = m->virtual_scr.VxMax + m->virtual_scr.MyDisplayWidth;
-			m->virtual_scr.VHeight = m->virtual_scr.VyMax + m->virtual_scr.MyDisplayHeight;
-			m->virtual_scr.VxPages = m->virtual_scr.VWidth / m->virtual_scr.MyDisplayWidth;
-			m->virtual_scr.VyPages = m->virtual_scr.VHeight / m->virtual_scr.MyDisplayHeight;
+			m->virtual_scr.VWidth = m->virtual_scr.VxMax + fpmonitor_get_all_widths();
+			m->virtual_scr.VHeight = m->virtual_scr.VyMax + fpmonitor_get_all_heights();
+			m->virtual_scr.VxPages = m->virtual_scr.VWidth / fpmonitor_get_all_widths();
+			m->virtual_scr.VyPages = m->virtual_scr.VHeight / fpmonitor_get_all_heights();
 		}
+	} else if (StrEquals(token, "DesktopConfiguration")) {
+		int mmode, is_shared;
+
+		sscanf(tline, "%d %d", &mmode, &is_shared);
+
+		if (mmode > 0)
+			monitor_mode = mmode;
 	}
 }
 
@@ -1685,7 +1731,10 @@ static void ParseColorset(char *arg1, char *arg2, void *offset_deskinfo,
   int desk;
   unsigned long colorset_offset = (unsigned long)offset_deskinfo;
   unsigned long item_colorset_offset = (unsigned long)offset_item;
-  struct fpmonitor *mon = fpmonitor_this();
+  struct fpmonitor *fp = fpmonitor_this(NULL);
+
+  if (fp == NULL)
+    return;
 
   sscanf(arg2, "%d", &colorset);
   AllocColorset(colorset);
@@ -1721,7 +1770,7 @@ static void ParseColorset(char *arg1, char *arg2, void *offset_deskinfo,
 	*(int *)(((char *)item) + item_colorset_offset) = colorset;
       }
     }
-    if (desk == mon->virtual_scr.CurrentDesk || all_desks)
+    if (desk == fp->m->virtual_scr.CurrentDesk || all_desks)
     {
       *(int *)(((char *)&Desks[0]) + colorset_offset) = colorset;
     }
@@ -1750,12 +1799,8 @@ static void SetDeskLabel(struct fpmonitor *m, int desk, const char *label)
     item = FindDeskStrings(desk);
     if (item->next != NULL)
     {
-      /* replace label */
-      if (item->next->label != NULL)
-      {
-	free(item->next->label);
-	item->next->label = NULL;
-      }
+      free(item->next->label);
+      item->next->label = NULL;
       CopyString(&(item->next->label), label);
     }
     else
@@ -1764,7 +1809,7 @@ static void SetDeskLabel(struct fpmonitor *m, int desk, const char *label)
       item = NewPagerStringItem(item, desk);
       CopyString(&(item->label), label);
     }
-    if (desk == m->virtual_scr.CurrentDesk)
+    if (desk == m->m->virtual_scr.CurrentDesk)
     {
       free(Desks[0].label);
       CopyString(&Desks[0].label, label);
@@ -1815,7 +1860,7 @@ void ParseOptions(void)
     char *token;
     char *next;
     Bool MoveThresholdSetForModule = False;
-    struct fpmonitor	*m = NULL, *m2 = NULL;
+    struct fpmonitor	*m = NULL;
 
     arg1 = arg2 = NULL;
 
@@ -1840,11 +1885,8 @@ void ParseOptions(void)
     }
     else if (StrEquals(token, "ImagePath"))
     {
-      if (ImagePath != NULL)
-      {
-	free(ImagePath);
-	ImagePath = NULL;
-      }
+      free(ImagePath);
+ImagePath = NULL;
       GetNextToken(next, &ImagePath);
 
       continue;
@@ -1875,30 +1917,6 @@ void ParseOptions(void)
       }
       continue;
     }
-    else if (StrEquals(token, "Monitor")) {
-	    char	*mname;
-	    int		 updated = 0;
-
-	    next = GetNextToken(next, &mname);
-
-	    TAILQ_FOREACH(m2, &fp_monitor_q, entry) {
-		    updated = 0;
-		    if (strcmp(m2->name, mname) == 0) {
-			    extract_monitor_config(m2, next);
-			    updated = 1;
-		    }
-	    }
-
-	    if (updated)
-		    continue;
-
-	    m = fxcalloc(1, sizeof(*m));
-
-	    m->name = fxstrdup(mname);
-	    extract_monitor_config(m, next);
-	    TAILQ_INSERT_TAIL(&fp_monitor_q, m, entry);
-	    continue;
-    }
 
     tline2 = GetModuleResource(tline, &resource, MyName);
     if (!resource)
@@ -1917,23 +1935,37 @@ void ParseOptions(void)
     }
 
     if (StrEquals(resource, "Monitor")) {
+	    struct monitor *tm = monitor_get_current();
+
 	    free(monitor_to_track);
+	    next = SkipSpaces(next, NULL, 0);
+
 	    if (next == NULL) {
 		    fvwm_debug(__func__, "FvwmPager: no monitor name given "
 				    "using current monitor\n");
 		    /* m already set... */
-		    monitor_to_track = fxstrdup(m->name);
+		    monitor_to_track = fxstrdup(tm->si->name);
 		    continue;
 	    }
 	    if ((m = fpmonitor_by_name(next)) == NULL) {
 		    fvwm_debug(__func__, "FvwmPager: monitor '%s' not found "
                                "using current monitor", next);
-		    m = fpmonitor_get_current();
-		    monitor_to_track = fxstrdup(m->name);
+		    monitor_to_track = fxstrdup(tm->si->name);
 		    continue;
 	    }
-	    fvwm_debug(__func__, "Assigning monitor: %s\n", m->name);
-	    monitor_to_track = fxstrdup(m->name);
+	    fvwm_debug(__func__, "Assigning monitor: %s\n", m->m->si->name);
+	    monitor_to_track = fxstrdup(m->m->si->name);
+	    if (preferred_monitor != NULL)
+		free(preferred_monitor);
+	    preferred_monitor = fxstrdup(monitor_to_track);
+    } else if(StrEquals(resource, "DeskLabels")) {
+	    use_desk_label = True;
+    } else if(StrEquals(resource, "NoDeskLabels")) {
+	    use_desk_label = False;
+    } else if(StrEquals(resource, "MonitorLabels")) {
+	    use_monitor_label = True;
+    } else if(StrEquals(resource, "NoMonitorLabels")) {
+	    use_monitor_label = False;
     }
     else if(StrEquals(resource,"Colorset"))
     {
@@ -2025,25 +2057,21 @@ void ParseOptions(void)
     }
     else if (StrEquals(resource, "Font"))
     {
-      if (font_string)
-	free(font_string);
+      free(font_string);
       CopyStringWithQuotes(&font_string, next);
       if(strncasecmp(font_string,"none",4) == 0)
       {
-	uselabel = 0;
-	if (font_string)
-	{
-	  free(font_string);
-	  font_string = NULL;
-	}
+	use_desk_label = False;
+	use_monitor_label = False;
+	free(font_string);
+	font_string = NULL;
       }
     }
     else if (StrEquals(resource, "Fore"))
     {
       if(Pdepth > 1)
       {
-	if (PagerFore)
-	  free(PagerFore);
+	free(PagerFore);
 	CopyString(&PagerFore,arg1);
       }
     }
@@ -2051,8 +2079,7 @@ void ParseOptions(void)
     {
       if(Pdepth > 1)
       {
-	if (PagerBack)
-	  free(PagerBack);
+	free(PagerBack);
 	CopyString(&PagerBack,arg1);
       }
     }
@@ -2074,12 +2101,8 @@ void ParseOptions(void)
 	item = FindDeskStrings(desk);
 	if (item->next != NULL)
 	{
-	  /* replace Dcolor */
-	  if (item->next->Dcolor != NULL)
-	  {
-	    free(item->next->Dcolor);
-	    item->next->Dcolor = NULL;
-	  }
+	  free(item->next->Dcolor);
+	  item->next->Dcolor = NULL;
 	  CopyString(&(item->next->Dcolor), arg2);
 	}
 	else
@@ -2088,9 +2111,9 @@ void ParseOptions(void)
 	  item = NewPagerStringItem(item, desk);
 	  CopyString(&(item->Dcolor), arg2);
 	}
-	m = fpmonitor_this();
+	struct fpmonitor *fp = fpmonitor_this(NULL);
 
-	if (desk == m->virtual_scr.CurrentDesk)
+	if (desk == fp->m->virtual_scr.CurrentDesk)
 	{
 	  free(Desks[0].Dcolor);
 	  CopyString(&Desks[0].Dcolor, arg2);
@@ -2136,8 +2159,8 @@ void ParseOptions(void)
 	  item->bgPixmap = PCacheFvwmPicture(
 		  dpy, Scr.Pager_w, ImagePath, arg2, fpa);
 	}
-	m = fpmonitor_this();
-	if (desk == m->virtual_scr.CurrentDesk)
+	struct fpmonitor *fp = fpmonitor_this(NULL);
+	if (desk == fp->m->virtual_scr.CurrentDesk)
 	{
 	  if (Desks[0].bgPixmap != NULL)
 	  {
@@ -2203,15 +2226,13 @@ void ParseOptions(void)
     {
       if(Pdepth > 1)
       {
-	if (HilightC)
-	  free(HilightC);
+	free(HilightC);
 	CopyString(&HilightC,arg1);
       }
     }
     else if (StrEquals(resource, "SmallFont"))
     {
-      if (smallFont)
-	free(smallFont);
+      free(smallFont);
       CopyStringWithQuotes(&smallFont, next);
       if (strncasecmp(smallFont,"none",4) == 0)
       {
@@ -2263,14 +2284,10 @@ void ParseOptions(void)
     {
       if (Pdepth > 1)
       {
-	if (WindowFore)
-	  free(WindowFore);
-	if (WindowBack)
-	  free(WindowBack);
-	if (WindowHiFore)
-	  free(WindowHiFore);
-	if (WindowHiBack)
-	  free(WindowHiBack);
+	free(WindowFore);
+	free(WindowBack);
+	free(WindowHiFore);
+	free(WindowHiBack);
 	CopyString(&WindowFore, arg1);
 	CopyString(&WindowBack, arg2);
 	tline2 = GetNextToken(tline2, &WindowHiFore);
@@ -2311,8 +2328,7 @@ void ParseOptions(void)
     }
     else if (StrEquals(resource,"WindowLabelFormat"))
     {
-      if (WindowLabelFormat)
-	free(WindowLabelFormat);
+      free(WindowLabelFormat);
       CopyString(&WindowLabelFormat,arg1);
     }
     else if (StrEquals(resource,"UseSkipList"))
@@ -2345,8 +2361,7 @@ void ParseOptions(void)
        -- ric@giccs.georgetown.edu */
     else if (StrEquals(resource, "Balloons"))
     {
-      if (BalloonTypeString)
-	free(BalloonTypeString);
+      free(BalloonTypeString);
       CopyString(&BalloonTypeString, arg1);
 
       if ( strncasecmp(BalloonTypeString, "Pager", 5) == 0 ) {
@@ -2372,8 +2387,7 @@ void ParseOptions(void)
     {
       if (Pdepth > 1)
       {
-	if (BalloonBack)
-	  free(BalloonBack);
+	free(BalloonBack);
 	CopyString(&BalloonBack, arg1);
       }
     }
@@ -2382,23 +2396,20 @@ void ParseOptions(void)
     {
       if (Pdepth > 1)
       {
-	if (BalloonFore)
-	  free(BalloonFore);
+	free(BalloonFore);
 	CopyString(&BalloonFore, arg1);
       }
     }
 
     else if (StrEquals(resource, "BalloonFont"))
     {
-      if (BalloonFont)
-	free(BalloonFont);
+      free(BalloonFont);
       CopyStringWithQuotes(&BalloonFont, next);
     }
 
     else if (StrEquals(resource, "BalloonBorderColor"))
     {
-      if (BalloonBorderColor)
-	free(BalloonBorderColor);
+      free(BalloonBorderColor);
       CopyString(&BalloonBorderColor, arg1);
     }
 
@@ -2424,8 +2435,7 @@ void ParseOptions(void)
     }
     else if (StrEquals(resource,"BalloonStringFormat"))
     {
-      if (BalloonFormatString)
-	free(BalloonFormatString);
+      free(BalloonFormatString);
       CopyString(&BalloonFormatString,arg1);
     }
 
@@ -2436,20 +2446,20 @@ void ParseOptions(void)
 
   struct fpmonitor *m;
   TAILQ_FOREACH(m, &fp_monitor_q, entry) {
-	  m->virtual_scr.VxMax = dx * m->virtual_scr.MyDisplayWidth - m->virtual_scr.MyDisplayWidth;
-	  m->virtual_scr.VyMax = dy * m->virtual_scr.MyDisplayHeight - m->virtual_scr.MyDisplayHeight;
+	  m->virtual_scr.VxMax = dx * fpmonitor_get_all_widths() - fpmonitor_get_all_widths();
+	  m->virtual_scr.VyMax = dy * fpmonitor_get_all_heights() - fpmonitor_get_all_heights();
 	  if (m->virtual_scr.VxMax < 0)
 		  m->virtual_scr.VxMax = 0;
 	  if (m->virtual_scr.VyMax < 0)
 		  m->virtual_scr.VyMax = 0;
-	  m->virtual_scr.VWidth = m->virtual_scr.VxMax + m->virtual_scr.MyDisplayWidth;
-	  m->virtual_scr.VHeight = m->virtual_scr.VyMax + m->virtual_scr.MyDisplayHeight;
-	  m->virtual_scr.VxPages = m->virtual_scr.VWidth / m->virtual_scr.MyDisplayWidth;
-	  m->virtual_scr.VyPages = m->virtual_scr.VHeight / m->virtual_scr.MyDisplayHeight;
+	  m->virtual_scr.VWidth = m->virtual_scr.VxMax + fpmonitor_get_all_widths();
+	  m->virtual_scr.VHeight = m->virtual_scr.VyMax + fpmonitor_get_all_heights();
+	  m->virtual_scr.VxPages = m->virtual_scr.VWidth / fpmonitor_get_all_widths();
+	  m->virtual_scr.VyPages = m->virtual_scr.VHeight / fpmonitor_get_all_heights();
 
 	  fvwm_debug(__func__,
 			  "%s: VxMax: %d, VyMax: %d, VWidth: %d, Vheight: %d, "
-			  "VxPages: %d, VyPages: %d\n", m->name,
+			  "VxPages: %d, VyPages: %d\n", m->m->si->name,
 			  m->virtual_scr.VxMax, m->virtual_scr.VyMax, m->virtual_scr.VWidth,
 			  m->virtual_scr.VHeight, m->virtual_scr.VxPages, m->virtual_scr.VyPages);
 
@@ -2490,6 +2500,14 @@ PagerStringList *NewPagerStringItem(PagerStringList *last, int desk)
 
 void ExitPager(void)
 {
+  struct fpmonitor *fp, *fp1;
+
+  TAILQ_FOREACH_SAFE(fp, &fp_monitor_q, entry, fp1) {
+	TAILQ_REMOVE(&fp_monitor_q, fp, entry);
+	free(fp->CPagerWin);
+	free(fp);
+  }
+
   if (is_transient)
   {
     XUngrabPointer(dpy,CurrentTime);
@@ -2497,5 +2515,7 @@ void ExitPager(void)
     XSync(dpy,0);
   }
   XUngrabKeyboard(dpy, CurrentTime);
+  free(monitor_to_track);
+  free(preferred_monitor);
   exit(0);
 }
